from abjad.tools.mathtools.NonreducedRatio import NonreducedRatio


class Ratio(NonreducedRatio):
    '''.. versionadded:: 2.10

    Ratio of one or more nonzero integers.

    Initialize from one or more nonzero integers::

        >>> mathtools.Ratio(2, 4, 2)
        Ratio(1, 2, 1)

    Or initialize from a tuple or list::

        >>> ratio = mathtools.Ratio((2, 4, 2))
        >>> ratio
        Ratio(1, 2, 1)

    Use a tuple to return ratio integers.

        >>> tuple(ratio)
        (1, 2, 1)

    Ratios are immutable.
    '''

    ### INITIALIZER ###

    def __new__(klass, *args):
        from abjad.tools import sequencetools
        if len(args) == 1 and isinstance(args[0], (list, tuple)):
            args = args[0]
        assert args, repr(args)
        assert all([x != 0 for x in args]), repr(args) 
        args = sequencetools.divide_sequence_elements_by_greatest_common_divisor(args)
        self = NonreducedRatio.__new__(klass, args)
        return self



from abjad.tools import beamtools
from abjad.tools import containertools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import notetools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker


class EvenRunRhythmMaker(RhythmMaker):
    r'''.. versionadded:: 2.11

    Even run rhythm-maker.

    Example 1. Make even run of notes each equal in duration to ``1/d``
    with ``d`` equal to the denominator of each division on which
    the rhythm-maker is called::

        >>> maker = rhythmmakertools.EvenRunRhythmMaker()

    ::

        >>> divisions = [(4, 16), (3, 8), (2, 8)]
        >>> lists = maker(divisions)
        >>> containers = sequencetools.flatten_sequence(lists)

    ::

        >>> staff = Staff(containers)

    ::

        >>> f(staff)
        \new Staff {
            {
                c'16 [
                c'16
                c'16
                c'16 ]
            }
            {
                c'8 [
                c'8
                c'8 ]
            }
            {
                c'8 [
                c'8 ]
            }
        }

    Example 2. Make even run of notes each equal in duration to ``1/(2**d)``
    with ``d`` equal to the denominator of each division on which
    the rhythm-maker is called::

        >>> maker = rhythmmakertools.EvenRunRhythmMaker(1)

    ::

        >>> divisions = [(4, 16), (3, 8), (2, 8)]
        >>> lists = maker(divisions)
        >>> containers = sequencetools.flatten_sequence(lists)

    ::

        >>> staff = Staff(containers)

    ::

        >>> f(staff)
        \new Staff {
            {
                c'32 [
                c'32
                c'32
                c'32
                c'32
                c'32
                c'32
                c'32 ]
            }
            {
                c'16 [
                c'16
                c'16
                c'16
                c'16
                c'16 ]
            }
            {
                c'16 [
                c'16
                c'16
                c'16 ]
            }
        }

    Output is a list (eventually a selection) of lists of depth-2 note-bearing containers .

    .. note:: doesn't yet work with non-power-of-two divisions.

    Usage follows the two-step instantiate-then-call pattern shown here.

    Return rhythm-maker.
    '''

    ### CLASS ATTRIBUTES ###

    _default_positional_input_arguments = ()

    ### INITIALIZER ###

    def __init__(self, denominator_multiplier_exponent=0, beam_each_cell=True, beam_cells_together=False):
        assert mathtools.is_nonnegative_integer(denominator_multiplier_exponent)
        RhythmMaker.__init__(self,
            beam_each_cell=beam_each_cell,
            beam_cells_together=beam_cells_together
            )
        self._denominator_multiplier_exponent = denominator_multiplier_exponent

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        result = []
        for division in divisions:
            container = self._make_container(division)
            result.append([container])
        return result

    ### PRIVATE METHODS ###

    def _make_container(self, division):
        numerator, denominator = division
        # eventually allow for non-power-of-two divisions
        assert mathtools.is_positive_integer_power_of_two(denominator)
        denominator_multiplier = 2 ** self.denominator_multiplier_exponent
        denominator *= denominator_multiplier
        unit_duration = durationtools.Duration(1, denominator)
        numerator *= denominator_multiplier
        notes = notetools.make_notes(numerator * [0], [unit_duration])
        container = containertools.Container(notes)
        if self.beam_each_cell:
            beam_spanner = beamtools.BeamSpanner()
            beam_spanner(container)
        return container

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def denominator_multiplier_exponent(self):
        '''Denominator multiplier exponent provided at initialization.
        
        Return nonnegative integer.
        '''
        return self._denominator_multiplier_exponent



from abjad.tools import mathtools


def yield_durations(unique=False):
    r'''.. versionadded:: 2.0

    Example 1. Yield all positive durations in Cantor diagonalized order::

        >>> generator = durationtools.yield_durations()
        >>> for n in range(16):
        ...     generator.next()
        ...
        Duration(1, 1)
        Duration(2, 1)
        Duration(1, 2)
        Duration(1, 3)
        Duration(1, 1)
        Duration(3, 1)
        Duration(4, 1)
        Duration(3, 2)
        Duration(2, 3)
        Duration(1, 4)
        Duration(1, 5)
        Duration(1, 2)
        Duration(1, 1)
        Duration(2, 1)
        Duration(5, 1)
        Duration(6, 1)

    Example 2. Yield all positive durations in Cantor diagonalized order uniquely::

        >>> generator = durationtools.yield_durations(unique=True)
        >>> for n in range(16):
        ...     generator.next()
        ...
        Duration(1, 1)
        Duration(2, 1)
        Duration(1, 2)
        Duration(1, 3)
        Duration(3, 1)
        Duration(4, 1)
        Duration(3, 2)
        Duration(2, 3)
        Duration(1, 4)
        Duration(1, 5)
        Duration(5, 1)
        Duration(6, 1)
        Duration(5, 2)
        Duration(4, 3)
        Duration(3, 4)
        Duration(2, 5)

    Return generator.
    '''
    from abjad.tools import durationtools

    generator = mathtools.yield_nonreduced_fractions()
    while True:
        integer_pair = generator.next()
        duration = durationtools.Duration(integer_pair)
        if not unique:
            yield duration
        elif duration.pair == integer_pair:
            yield duration



from abjad.tools import mathtools


def spell_chromatic_interval_number(diatonic_interval_number, chromatic_interval_number):
    '''.. versionadded:: 2.0

    Spell `chromatic_interval_number` according to `diatonic_interval_number`::

        >>> pitchtools.spell_chromatic_interval_number(2, 1)
        MelodicDiatonicInterval('+m2')

    Return melodic diatonic interval.
    '''
    from abjad.tools import pitchtools

    if not isinstance(chromatic_interval_number, int):
        raise IntervalError('can not determine diatonic interval from float.')

    direction_number = mathtools.sign(chromatic_interval_number)

    if diatonic_interval_number == 1:
        if chromatic_interval_number % 12 == 11:
            quality_string = 'augmented'
        elif chromatic_interval_number % 12 == 0:
            quality_string = 'perfect'
        elif chromatic_interval_number % 12 == 1:
            quality_string = 'augmented'
        if not direction_number == 0:
            diatonic_interval_number *= direction_number
        diatonic_interval = pitchtools.MelodicDiatonicInterval(quality_string, diatonic_interval_number)
        return diatonic_interval

    diatonic_interval_class_number = diatonic_interval_number % 7
    chromatic_interval_class_number = abs(chromatic_interval_number) % 12

    if diatonic_interval_class_number == 0:
        if chromatic_interval_class_number == 9:
            quality_string = 'diminished'
        elif chromatic_interval_class_number == 10:
            quality_string = 'minor'
        elif chromatic_interval_class_number == 11:
            quality_string = 'major'
        elif chromatic_interval_class_number == 0:
            quality_string = 'augmented'
    elif diatonic_interval_class_number == 1:
        if chromatic_interval_class_number == 11:
            quality_string = 'diminished'
        elif chromatic_interval_class_number == 0:
            quality_string = 'perfect'
        elif chromatic_interval_class_number == 1:
            quality_string = 'augmented'
    elif diatonic_interval_class_number == 2:
        if chromatic_interval_class_number == 0:
            quality_string = 'diminished'
        elif chromatic_interval_class_number == 1:
            quality_string = 'minor'
        elif chromatic_interval_class_number == 2:
            quality_string = 'major'
        elif chromatic_interval_class_number == 3:
            quality_string = 'augmented'
    elif diatonic_interval_class_number == 3:
        if chromatic_interval_class_number == 2:
            quality_string = 'diminished'
        elif chromatic_interval_class_number == 3:
            quality_string = 'minor'
        elif chromatic_interval_class_number == 4:
            quality_string = 'major'
        elif chromatic_interval_class_number == 5:
            quality_string = 'augmented'
    elif diatonic_interval_class_number == 4:
        if chromatic_interval_class_number == 4:
            quality_string = 'diminished'
        elif chromatic_interval_class_number == 5:
            quality_string = 'perfect'
        elif chromatic_interval_class_number == 6:
            quality_string = 'augmented'
    elif diatonic_interval_class_number == 5:
        if chromatic_interval_class_number == 6:
            quality_string = 'diminished'
        elif chromatic_interval_class_number == 7:
            quality_string = 'perfect'
        elif chromatic_interval_class_number == 8:
            quality_string = 'augmented'
    elif diatonic_interval_class_number == 6:
        if chromatic_interval_class_number == 7:
            quality_string = 'diminished'
        elif chromatic_interval_class_number == 8:
            quality_string = 'minor'
        elif chromatic_interval_class_number == 9:
            quality_string = 'major'
        elif chromatic_interval_class_number == 10:
            quality_string = 'augmented'
    elif diatonic_interval_class_number == 7:
        if chromatic_interval_class_number == 9:
            quality_string = 'diminished'
        elif chromatic_interval_class_number == 10:
            quality_string = 'minor'
        elif chromatic_interval_class_number == 11:
            quality_string = 'major'
        elif chromatic_interval_class_number == 0:
            quality_string = 'augmented'
    elif diatonic_interval_class_number == 8:
        if chromatic_interval_class_number == 11:
            quality_string = 'diminished'
        elif chromatic_interval_class_number == 0:
            quality_string = 'perfect'
        elif chromatic_interval_class_number == 1:
            quality_string = 'augmented'

    if not direction_number == 0:
        diatonic_interval_number *= direction_number

    diatonic_interval = pitchtools.MelodicDiatonicInterval(quality_string, diatonic_interval_number)

    return diatonic_interval



import fractions
from abjad.tools import durationtools
from abjad.tools.timerelationtools.TimeRelation import TimeRelation


class OffsetTimespanTimeRelation(TimeRelation):
    r'''.. versionadded:: 2.11

    Offset / timespan time relation:

    ::

        >>> offset = Offset(5)
        >>> timespan = timespantools.Timespan(0, 10)
        >>> time_relation = timerelationtools.offset_happens_during_timespan(
        ...     offset=offset, timespan=timespan, hold=True)

    ::

        >>> z(time_relation)
        timerelationtools.OffsetTimespanTimeRelation(
            'timespan.start <= offset < timespan.stop',
            timespan=timespantools.Timespan(
                start_offset=durationtools.Offset(0, 1),
                stop_offset=durationtools.Offset(10, 1)
                ),
            offset=durationtools.Offset(5, 1)
            )

    Offset / timespan time relations are immutable.
    '''

    ### INITIALIZER ###

    def __init__(self, template, timespan=None, offset=None):
        TimeRelation.__init__(self, template)
        self._timespan = timespan
        self._offset = offset

    ### SPECIAL METHODS ###

    def __call__(self, timespan=None, offset=None):
        r'''Evaluate time relation:

            >>> time_relation()
            True

        Raise value error is either `offset` or `timespan` is none.

        Otherwise return boolean.
        '''
        from abjad.tools import timespantools
        if timespan is None:
            timespan = self.timespan
        if offset is None:
            offset = self.offset
        if timespan is None or offset is None:
            raise ValueError('time relation is not fully loaded.')
        timespan = timespantools.Timespan()._get_timespan(timespan)
        offset = durationtools.Offset(offset)
        timespan_start, timespan_stop = self._get_expr_offsets(timespan)
        command = self.template
        command = command.replace('timespan.start', repr(timespan_start))
        command = command.replace('timespan.stop', repr(timespan_stop))
        command = command.replace('offset', repr(offset))
        result = eval(command, {'Offset': durationtools.Offset})
        return result

    def __eq__(self, expr):
        '''True when `expr` equals time relation. Otherwise false:

        ::

            >>> offset = Offset(5)
            >>> time_relation_1 = timerelationtools.offset_happens_during_timespan()
            >>> time_relation_2 = timerelationtools.offset_happens_during_timespan(
            ...     offset=offset)

        ::

            >>> time_relation_1 == time_relation_1
            True
            >>> time_relation_1 == time_relation_2
            False 
            >>> time_relation_2 == time_relation_2
            True

        Return boolean.
        '''
        if isinstance(expr, type(self)):
            if self.template == expr.template:
                if self.timespan == expr.timespan:
                    if self.offset == expr.offset:
                        return True
        return False

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def is_fully_loaded(self):
        '''True when `timespan` and `offset` are both not none.
        Otherwise false:

        ::

            >>> time_relation.is_fully_loaded
            True

        Return boolean.
        '''
        return self.timespan is not None and self.offset is not None

    @property
    def is_fully_unloaded(self):
        '''True when `timespan` and `offset` are both none.
        Otherwise false:
    
        ::

            >>> time_relation.is_fully_unloaded
            False

        Return boolean.
        '''
        return self.timespan is None and self.offset is None

    @property
    def offset(self):
        '''Time relation offset:

        ::

            >>> time_relation.offset
            Offset(5, 1)

        Return offset or none.
        '''
        return self._offset

    @property
    def storage_format(self):
        '''Time relation storage format:

        ::

            >>> z(time_relation)
            timerelationtools.OffsetTimespanTimeRelation(
                'timespan.start <= offset < timespan.stop',
                timespan=timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    ),
                offset=durationtools.Offset(5, 1)
                )

        Return string.
        '''
        return TimeRelation.storage_format.fget(self)

    @property
    def timespan(self):
        '''Time relation timepsan:

        ::

            >>> time_relation.timespan
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(10, 1))

        Return timespan or none.
        '''
        return self._timespan



import itertools
from abjad.tools import chordtools
from abjad.tools import componenttools
from abjad.tools import notetools
from abjad.tools import sequencetools
from abjad.tools import spannertools
from abjad.tools.selectiontools.Selection import Selection


class TieChain(Selection):
    '''.. versionadded:: 2.9

    All the notes in a tie chain::

        >>> staff = Staff("c' d' e' ~ e'")

    ::

        >>> tietools.get_tie_chain(staff[2])
        TieChain(Note("e'4"), Note("e'4"))

    Tie chains are immutable score selections.
    '''

    def __init__(self, music):
        if isinstance(music, Selection):
            Selection.__init__(self, music.music)
        else:
            Selection.__init__(self, music)

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def all_leaves_are_in_same_parent(self):
        '''True when all leaves in tie chain are in same parent.

        Return boolean.
        '''
        return sequencetools.all_are_equal([leaf.parent for leaf in self.leaves])

    @property
    def duration_in_seconds(self):
        '''Read-only duration in seconds of components in tie chain.

        Return duration.
        '''
        return sum([x.duration_in_seconds for x in self])

    @property
    def head(self):
        '''Read-only reference to element ``0`` in tie chain.
        '''
        if self.music:
            return self.music[0]

    @property
    def is_pitched(self):
        '''True when tie chain head is a note or chord.

        Return boolean.
        '''
        return isinstance(self.head, (notetools.Note, chordtools.Chord))

    @property
    def is_trivial(self):
        '''True when length of tie chain is less than or equal to ``1``.

        Return boolean.
        '''
        return len(self) <= 1

    @property
    def leaves(self):
        '''Read-only tuple of leaves in tie spanner.
        '''
        from abjad.tools import tietools
        try:
            tie_spanner = spannertools.get_the_only_spanner_attached_to_component(self[0], tietools.TieSpanner)
            return tie_spanner.leaves
        except MissingSpannerError:
            assert self.is_trivial
            return (self[0], )

    @property
    def leaves_grouped_by_immediate_parents(self):
        '''Read-only list of leaves in tie chain grouped by immediate parents of leaves.

        Return list of lists.
        '''
        result = []
        pairs_generator = itertools.groupby(self, lambda x: id(x.parent))
        for key, values_generator in pairs_generator:
            result.append(list(values_generator))
        return result

    @property
    def preprolated_duration(self):
        '''Sum of preprolated durations of all components in tie chain.
        '''
        return sum([x.preprolated_duration for x in self])

    @property
    def prolated_duration(self):
        '''Sum of prolated durations of all components in tie chain.
        '''
        return sum([x.prolated_duration for x in self])

    @property
    def written_duration(self):
        '''Sum of written duration of all components in tie chain.
        '''
        return sum([x.written_duration for x in self])



from abjad.tools import leaftools


def iterate_leaves_in_expr(expr, reverse=False, start=0, stop=None):
    r'''.. versionadded:: 2.10

    Iterate leaves forward in `expr`::

        >>> staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
        }

    ::

        >>> for leaf in iterationtools.iterate_leaves_in_expr(staff):
        ...     leaf
        ...
        Note("c'8")
        Note("d'8")
        Note("e'8")
        Note("f'8")
        Note("g'8")
        Note("a'8")

    Use the optional `start` and `stop` keyword parameters to control
    the start and stop indices of iteration. ::

        >>> for leaf in iterationtools.iterate_leaves_in_expr(staff, start=3):
        ...     leaf
        ...
        Note("f'8")
        Note("g'8")
        Note("a'8")

    ::

        >>> for leaf in iterationtools.iterate_leaves_in_expr(staff, start=0, stop=3):
        ...     leaf
        ...
        Note("c'8")
        Note("d'8")
        Note("e'8")

    ::

        >>> for leaf in iterationtools.iterate_leaves_in_expr(staff, start=2, stop=4):
        ...     leaf
        ...
        Note("e'8")
        Note("f'8")

    Iterate leaves backward in `expr`:

    ::

        >>> for leaf in iterationtools.iterate_leaves_in_expr(staff, reverse=True):
        ...     leaf
        ...
        Note("a'8")
        Note("g'8")
        Note("f'8")
        Note("e'8")
        Note("d'8")
        Note("c'8")

    ::

        >>> for leaf in iterationtools.iterate_leaves_in_expr(
        ...     staff, start=3, reverse=True):
        ...     leaf
        ...
        Note("e'8")
        Note("d'8")
        Note("c'8")

    ::

        >>> for leaf in iterationtools.iterate_leaves_in_expr(
        ...     staff, start=0, stop=3, reverse=True):
        ...     leaf
        ...
        Note("a'8")
        Note("g'8")
        Note("f'8")

    ::

        >>> for leaf in iterationtools.iterate_leaves_in_expr(
        ...     staff, start=2, stop=4, reverse=True):
        ...     leaf
        ...
        Note("f'8")
        Note("e'8")

    Ignore threads.

    Return generator.
    '''
    from abjad.tools import iterationtools

    return iterationtools.iterate_components_in_expr(
        expr, klass=leaftools.Leaf, reverse=reverse, start=start, stop=stop)




import itertools
from abjad.tools import componenttools
from abjad.tools import contexttools
from abjad.tools import durationtools
from abjad.tools import notetools
from abjad.tools import resttools
from abjad.tools import skiptools
from abjad.tools import tietools


def tempo_scaled_leaves_to_q_events(leaves, tempo=None):
    '''Convert `leaves` to a list of ``QEvent`` objects.
    If the leaves have no effective tempo, `tempo` must be a
    ``TempoMark``.

    ::

        >>> source = Staff("c'4 r4. e'8 <g' b' d''>2")
        >>> source_tempo = contexttools.TempoMark((1, 4), 55)
        >>> for x in quantizationtools.tempo_scaled_leaves_to_q_events(
        ...     source[:], tempo=source_tempo): x
        ...
        quantizationtools.PitchedQEvent(
            durationtools.Offset(0, 1),
            (NamedChromaticPitch("c'"),),
            attachments=()
            )
        quantizationtools.SilentQEvent(
            durationtools.Offset(12000, 11),
            attachments=()
            )
        quantizationtools.PitchedQEvent(
            durationtools.Offset(30000, 11),
            (NamedChromaticPitch("e'"),),
            attachments=()
            )
        quantizationtools.PitchedQEvent(
            durationtools.Offset(36000, 11),
            (NamedChromaticPitch("g'"), NamedChromaticPitch("b'"), NamedChromaticPitch("d''")),
            attachments=()
            )
        quantizationtools.TerminalQEvent(
            durationtools.Offset(60000, 11)
            )

    Return a list of ``QEvent`` objects.
    '''
    from abjad.tools import quantizationtools

    assert componenttools.all_are_contiguous_components_in_same_thread(leaves) and len(leaves)
    if tempo is None:
        assert contexttools.get_effective_tempo(leaves[0]) is not None
    else:
        tempo = contexttools.TempoMark(tempo)

    # sort by silence and tied leaves
    groups = []
    for rvalue, rgroup in itertools.groupby(
        leaves, 
        lambda x: isinstance(x, (resttools.Rest, skiptools.Skip))):
        if rvalue:
            groups.append(list(rgroup))
        else:
            for tvalue, tgroup in itertools.groupby(rgroup, lambda x: tietools.get_tie_chain(x)):
                groups.append(list(tgroup))

    # calculate lists of pitches and durations
    durations = []
    pitches = []
    for group in groups:

        # get millisecond cumulative duration
        if tempo is not None:
            duration = sum([quantizationtools.tempo_scaled_duration_to_milliseconds(
                x.prolated_duration, tempo)
                for x in group])
        else:
            duration = sum([quantizationtools.tempo_scaled_duration_to_milliseconds(x.prolated_duration,
                contexttools.get_effective_tempo(x)) for x in group])
        durations.append(duration)

        # get pitch of first leaf in group
        if isinstance(group[0], (resttools.Rest, skiptools.Skip)):
            pitch = None
        elif isinstance(group[0], notetools.Note):
            pitch = group[0].written_pitch.chromatic_pitch_number
        else: # chord
            pitch = [x.written_pitch.chromatic_pitch_number for x in group[0].note_heads]
        pitches.append(pitch)

    # convert durations and pitches to QEvents and return
    return quantizationtools.millisecond_pitch_pairs_to_q_events(zip(durations, pitches))



from abjad import *


def test_Timespan_overlaps_stop_of_timespan_01():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(-10, -5)
    assert not a.overlaps_stop_of_timespan(b)

def test_Timespan_overlaps_stop_of_timespan_02():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(-10, 0)
    assert not a.overlaps_stop_of_timespan(b)

def test_Timespan_overlaps_stop_of_timespan_03():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(-10, 5)
    assert a.overlaps_stop_of_timespan(b)

def test_Timespan_overlaps_stop_of_timespan_04():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(-10, 15)
    assert not a.overlaps_stop_of_timespan(b)

def test_Timespan_overlaps_stop_of_timespan_05():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(-10, 25)
    assert not a.overlaps_stop_of_timespan(b)

def test_Timespan_overlaps_stop_of_timespan_06():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(0, 10)
    assert a.overlaps_stop_of_timespan(b)

def test_Timespan_overlaps_stop_of_timespan_07():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(0, 15)
    assert not a.overlaps_stop_of_timespan(b)

def test_Timespan_overlaps_stop_of_timespan_08():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(5, 10)
    assert a.overlaps_stop_of_timespan(b)

def test_Timespan_overlaps_stop_of_timespan_09():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(5, 15)
    assert not a.overlaps_stop_of_timespan(b)

def test_Timespan_overlaps_stop_of_timespan_10():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(0, 25)
    assert not a.overlaps_stop_of_timespan(b)

def test_Timespan_overlaps_stop_of_timespan_11():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(5, 25)
    assert not a.overlaps_stop_of_timespan(b)

def test_Timespan_overlaps_stop_of_timespan_12():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(15, 25)
    assert not a.overlaps_stop_of_timespan(b)

def test_Timespan_overlaps_stop_of_timespan_13():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(20, 25)
    assert not a.overlaps_stop_of_timespan(b)





from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import notetools
from abjad.tools import resttools
from abjad.tools.rhythmtreetools.RhythmTreeNode import RhythmTreeNode


class RhythmTreeLeaf(RhythmTreeNode):
    '''A leaf node in a rhythm tree:

    ::

        >>> leaf = rhythmtreetools.RhythmTreeLeaf(duration=5, is_pitched=True)
        >>> leaf
        RhythmTreeLeaf(
            duration=Duration(5, 1),
            is_pitched=True
            )
    
    Call with a pulse duration to generate Abjad leaf objects:

    ::

        >>> result = leaf((1, 8))
        >>> result
        [Note("c'2"), Note("c'8")]

    Generates rests when called, if `is_pitched` is False:

    ::

        >>> rhythmtreetools.RhythmTreeLeaf(duration=7, is_pitched=False)((1, 16))
        [Rest('r4..')]

    Return `RhythmTreeLeaf`.
    '''

    ### INITIALIZER ###

    def __init__(self, duration=1, is_pitched=True, name=None):
        RhythmTreeNode.__init__(self, duration=duration, name=name)
        self.is_pitched = is_pitched

    ### SPECIAL METHODS ###

    def __call__(self, pulse_duration):
        '''Generate Abjad score components:

        ::

            >>> leaf = rhythmtreetools.RhythmTreeLeaf(5)
            >>> leaf((1, 4))
            [Note("c'1"), Note("c'4")]

        Return sequence of components.
        '''
        pulse_duration = durationtools.Duration(pulse_duration)
        total_duration = pulse_duration * self.duration
        if self.is_pitched:
            return notetools.make_notes(0, total_duration)
        return resttools.make_rests(total_duration)

    def __eq__(self, other):
        if type(self) == type(other):
            if self.duration == other.duration:
                if self.is_pitched == other.is_pitched:
                    return True
        return False

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _pretty_rtm_format_pieces(self):
        return [str(self.duration)]

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def graphviz_graph(self):
        graph = documentationtools.GraphvizGraph(name='G')
        node = documentationtools.GraphvizNode(
            attributes={
                label: str(self.duration),
                shape: 'box'
            }
            )
        graph.append(node)
        return graph

    @property
    def rtm_format(self):
        '''The node's RTM format:

        ::

            >>> rhythmtreetools.RhythmTreeLeaf(1, is_pitched=True).rtm_format
            '1'
            >>> rhythmtreetools.RhythmTreeLeaf(5, is_pitched=False).rtm_format
            '-5'

        Return string.
        '''
        if self.is_pitched:
            return '{}'.format(self.duration)
        return '-{}'.format(self.duration)

    ### READ/WRITE PUBLIC PROPERTIES ###

    @apply
    def is_pitched():
        def fget(self):
            '''True if leaf is pitched:

            ::

                >>> leaf = rhythmtreetools.RhythmTreeLeaf()
                >>> leaf.is_pitched
                True

            ::

                >>> leaf.is_pitched = False
                >>> leaf.is_pitched
                False

            Return boolean.
            '''
            return self._is_pitched
        def fset(self, arg):
            self._is_pitched = bool(arg)
        return property(**locals())




from abjad.tools import spannertools


def iterate_nontrivial_tie_chains_in_expr(expr, reverse=False):
    r'''Iterate nontrivial tie chains forward in `expr`::

        >>> staff = Staff(r"c'4 ~ \times 2/3 { c'16 d'8 } e'8 f'4 ~ f'16")

    ::

        >>> f(staff)
        \new Staff {
            c'4 ~
            \times 2/3 {
                c'16
                d'8
            }
            e'8
            f'4 ~
            f'16
        }

    ::

        >>> for x in tietools.iterate_nontrivial_tie_chains_in_expr(staff):
        ...     x
        ...
        TieChain(Note("c'4"), Note("c'16"))
        TieChain(Note("f'4"), Note("f'16"))

    Iterate nontrivial tie chains backward in `expr`::

        >>> for x in tietools.iterate_nontrivial_tie_chains_in_expr(staff, reverse=True):
        ...     x
        ...
        TieChain(Note("f'4"), Note("f'16"))
        TieChain(Note("c'4"), Note("c'16"))

    Return generator.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import tietools

    if not reverse:
        for leaf in iterationtools.iterate_leaves_in_expr(expr):
            tie_spanners = spannertools.get_spanners_attached_to_component(leaf, tietools.TieSpanner)
            if not tie_spanners or tuple(tie_spanners)[0]._is_my_last_leaf(leaf):
                tie_chain = tietools.get_tie_chain(leaf)
                if not tie_chain.is_trivial:
                    yield tie_chain
    else:
        for leaf in iterationtools.iterate_leaves_in_expr(expr, reverse=True):
            tie_spanners = spannertools.get_spanners_attached_to_component(leaf, tietools.TieSpanner)
            if not(tie_spanners) or tuple(tie_spanners)[0]._is_my_first_leaf(leaf):
                tie_chain = tietools.get_tie_chain(leaf)
                if not tie_chain.is_trivial:
                    yield tie_chain



from abjad.tools import durationtools
from abjad.tools.timerelationtools.TimeRelation import TimeRelation


class TimespanTimespanTimeRelation(TimeRelation):
    r'''.. versionadded:: 2.11

    Timespan / timespan time relation.

    Score for examples:

    ::

        >>> staff_1 = Staff(r"\times 2/3 { c'4 d'4 e'4 } \times 2/3 { f'4 g'4 a'4 }")
        >>> staff_2 = Staff("c'2. d'4")
        >>> score = Score([staff_1, staff_2])

    ::

        >>> f(score)
        \new Score <<
            \new Staff {
                \times 2/3 {
                    c'4
                    d'4
                    e'4
                }
                \times 2/3 {
                    f'4
                    g'4
                    a'4
                }
            }
            \new Staff {
                c'2.
                d'4
            }
        >>

    ::

        >>> last_tuplet = staff_1[-1]
        >>> long_note = staff_2[0]

    ::

        >>> show(score) # doctest: +SKIP

    Example functions calls using the score above:
    
    ::

        >>> timerelationtools.timespan_2_happens_during_timespan_1(
        ... timespan_1=last_tuplet, timespan_2=long_note)
        False

    ::

        >>> timerelationtools.timespan_2_intersects_timespan_1(
        ... timespan_1=last_tuplet, timespan_2=long_note)
        True

    ::

        >>> timerelationtools.timespan_2_is_congruent_to_timespan_1(
        ... timespan_1=last_tuplet, timespan_2=long_note)
        False

    ::

        >>> timerelationtools.timespan_2_overlaps_all_of_timespan_1(
        ... timespan_1=last_tuplet, timespan_2=long_note)
        False

    ::

        >>> timerelationtools.timespan_2_overlaps_start_of_timespan_1(
        ... timespan_1=last_tuplet, timespan_2=long_note)
        True

    ::

        >>> timerelationtools.timespan_2_overlaps_stop_of_timespan_1(
        ... timespan_1=last_tuplet, timespan_2=long_note)
        False

    ::

        >>> timerelationtools.timespan_2_starts_after_timespan_1_starts(
        ... timespan_1=last_tuplet, timespan_2=long_note)
        False

    ::

        >>> timerelationtools.timespan_2_starts_after_timespan_1_stops(
        ... timespan_1=last_tuplet, timespan_2=long_note)
        False

    Timespan / timespan time relations are immutable.
    '''

    ### INITIALIZER ###

    def __init__(self, template, timespan_1=None, timespan_2=None):
        TimeRelation.__init__(self, template)
        self._timespan_1 = timespan_1
        self._timespan_2 = timespan_2

    ### SPECIAL METHODS ###

    def __call__(self, timespan_1=None, timespan_2=None, score_specification=None, context_name=None):
        r'''Evaluate time relation.

        Example 1. Evaluate time relation without substitution:
    
        ::

            >>> timespan_1 = timespantools.Timespan(5, 15)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> time_relation = timerelationtools.timespan_2_starts_during_timespan_1(
            ...     timespan_1=timespan_1, timespan_2=timespan_2, hold=True)

        ::

            >>> z(time_relation)
            timerelationtools.TimespanTimespanTimeRelation(
                'timespan_1.start <= timespan_2.start < timespan_1.stop',
                timespan_1=timespantools.Timespan(
                    start_offset=durationtools.Offset(5, 1),
                    stop_offset=durationtools.Offset(15, 1)
                    ),
                timespan_2=timespantools.Timespan(
                    start_offset=durationtools.Offset(10, 1),
                    stop_offset=durationtools.Offset(20, 1)
                    )
                )

        ::

            >>> time_relation()
            True

        Example 2. Substitute `timespan_1` during evaluation:

        ::

            >>> new_timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> new_timespan_1
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(10, 1))

        ::

            >>> time_relation(timespan_1=new_timespan_1)
            False

        Example 3. Substitute `timespan_2` during evaluation:

        ::

            >>> new_timespan_2 = timespantools.Timespan(2, 12)

        ::

            >>> new_timespan_2
            Timespan(start_offset=Offset(2, 1), stop_offset=Offset(12, 1))

        ::

            >>> time_relation(timespan_2=new_timespan_2)
            False
        
        Example 4. Substitute both `timespan_1` and `timespan_2` during evaluation:

        ::

            >>> time_relation(timespan_1=new_timespan_1, timespan_2=new_timespan_2)
            True

        Raise value error if either `timespan_1` or `timespan_2` is none.

        Otherwise return boolean.
        '''
        from abjad.tools import timespantools
        if timespan_1 is None:
            timespan_1 = self.timespan_1
        if timespan_2 is None:
            timespan_2 = self.timespan_2
        if timespan_1 is None or timespan_2 is None:
            raise ValueError('time relation is not fully loaded: {!r}.'.format(self))
        timespan_1 = timespantools.Timespan()._get_timespan(timespan_1)
        timespan_2 = timespantools.Timespan()._get_timespan(timespan_2)
        timespan_1_start, timespan_1_stop = self._get_expr_offsets(
            timespan_1, score_specification=score_specification, context_name=context_name)
        timespan_2_start, timespan_2_stop = self._get_expr_offsets(
            timespan_2, score_specification=score_specification, context_name=context_name)
        command = self.template
        command = command.replace('timespan_1.start', repr(timespan_1_start))
        command = command.replace('timespan_1.stop', repr(timespan_1_stop))
        command = command.replace('timespan_2.start', repr(timespan_2_start))
        command = command.replace('timespan_2.stop', repr(timespan_2_stop))
        result = eval(command, {'Offset': durationtools.Offset})
        return result
    
    def __eq__(self, expr):
        '''True when `expr` equals time relation. Otherwise false:

        ::

            >>> timespan = timespantools.Timespan(0, 10)
            >>> time_relation_1 = timerelationtools.timespan_2_starts_during_timespan_1()
            >>> time_relation_2 = timerelationtools.timespan_2_starts_during_timespan_1(
            ...     timespan_1=timespan)

        ::

            >>> time_relation_1 == time_relation_1
            True
            >>> time_relation_1 == time_relation_2
            False 
            >>> time_relation_2 == time_relation_2
            True

        Return boolean.
        '''
        if isinstance(expr, type(self)):
            if self.template == expr.template:
                if self.timespan_1 == expr.timespan_1:
                    if self.timespan_2 == expr.timespan_2:
                        return True
        return False

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def is_fully_loaded(self):
        '''True when `timespan_1` and `timespan_2` are both not none.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)
            >>> time_relation = timerelationtools.timespan_2_starts_during_timespan_1(
            ...     timespan_1=timespan_1, timespan_2=timespan_2, hold=True)

        ::
    
            >>> time_relation.is_fully_loaded
            True

        Return boolean.
        '''
        return self.timespan_1 is not None and self.timespan_2 is not None

    @property
    def is_fully_unloaded(self):
        '''True when `timespan_1` and `timespan_2` are both none.
        Otherwise false.

            >>> time_relation.is_fully_unloaded
            False

        Return boolean.
        '''
        return self.timespan_1 is None and self.timespan_2 is None

    @property
    def timespan_1(self):
        '''Time relation timespan ``1``:

        ::

            >>> time_relation.timespan_1
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(10, 1))

        Return timespan.
        '''
        return self._timespan_1
        
    @property
    def timespan_2(self):
        '''Time relation timespan ``2``:

        ::

            >>> time_relation.timespan_2
            Timespan(start_offset=Offset(5, 1), stop_offset=Offset(15, 1))

        Return timespan.
        '''
        return self._timespan_2



from abjad import *


def test_MeasuredComplexBeamSpanner___init___01():
    '''Init empty measured complex beam spanner.
    '''

    beam = beamtools.MeasuredComplexBeamSpanner()
    assert isinstance(beam, beamtools.MeasuredComplexBeamSpanner)



import collections


def count_offsets_in_expr(expr):
    r'''Count offsets in `expr`.
    
    Example 1.:
    
    ::

        >>> score = Score()
        >>> score.append(Staff("c'4. d'8 e'2"))
        >>> score.append(Staff(r'\clef bass c4 b,4 a,2'))
        >>> f(score)
        \new Score <<
            \new Staff {
                c'4.
                d'8
                e'2
            }
            \new Staff {
                \clef "bass"
                c4
                b,4
                a,2
            }
        >>

    ::

        >>> counter = durationtools.count_offsets_in_expr(score.leaves)
        >>> for offset, count in sorted(counter.items()):
        ...     offset, count
        ...
        (Offset(0, 1), 2)
        (Offset(1, 4), 2)
        (Offset(3, 8), 2)
        (Offset(1, 2), 4)
        (Offset(1, 1), 2)

    Example 2.:

    ::

        >>> a = timespantools.Timespan(0, 10)
        >>> b = timespantools.Timespan(5, 15)
        >>> c = timespantools.Timespan(15, 20)

    ::

        >>> counter = durationtools.count_offsets_in_expr((a, b, c))
        >>> for offset, count in sorted(counter.items()):
        ...     offset, count
        ...
        (Offset(0, 1), 1)
        (Offset(5, 1), 1)
        (Offset(10, 1), 1)
        (Offset(15, 1), 2)
        (Offset(20, 1), 1)

    Return Counter.
    '''
    from abjad.tools import durationtools
    counter = collections.Counter()
    for x in expr:
        if hasattr(x, 'start_offset') and hasattr(x, 'stop_offset'):
            counter[x.start_offset] += 1
            counter[x.stop_offset] += 1
        elif hasattr(x, 'timespan'):
            counter[x.timespan.start_offset] += 1
            counter[x.timespan.stop_offset] += 1
        else:
            offset = durationtools.Offset(x)
            counter[offset] += 1
    return counter



from abjad.demos.ferneyhough.make_nested_tuplet import make_nested_tuplet


def make_row_of_nested_tuplets(tuplet_duration, outer_tuplet_proportions, column_count):
    assert 0 < column_count
    row_of_nested_tuplets = []
    for n in range(column_count):
        inner_tuplet_subdivision_count = n + 1
        nested_tuplet = make_nested_tuplet(
            tuplet_duration, outer_tuplet_proportions, inner_tuplet_subdivision_count)
        row_of_nested_tuplets.append(nested_tuplet)
    return row_of_nested_tuplets




from abjad import *
import py


def test_Measure_extend_01():
    '''Do not adjust time signature automatically.
    '''

    measure = Measure((3, 4), "c' d' e'")
    measure.extend("f' g'")

    assert measure.is_overfull
    assert py.test.raises(Exception, 'f(measure)')


def test_Measure_extend_02():
    '''Adjust time signature automatically.
    '''

    measure = Measure((3, 4), "c' d' e'")
    measure.automatically_adjust_time_signature = True
    measure.extend("f' g'")

    assert not measure.is_misfilled
    assert measure.lilypond_format == "{\n\t\\time 5/4\n\tc'4\n\td'4\n\te'4\n\tf'4\n\tg'4\n}"



import abc
from abjad.tools.documentationtools.Documenter import Documenter
import inspect


class ClassDocumenter(Documenter):
    '''ClassDocumenter generates an ReST API entry for a given class:

    ::

        >>> from abjad.tools.documentationtools import ClassDocumenter
    
    ::

        >>> documenter = ClassDocumenter(notetools.Note)
        >>> rest = documenter()

    Returns ``ClassDocumenter`` instance.
    '''

    ### CLASS ATTRIBUTES ###

    _ignored_special_methods = (
        '__copy__', '__deepcopy__', '__format__', '__getattribute__',
        '__getnewargs__', '__init__', '__reduce__', '__reduce_ex__', 
        '__sizeof__', '__subclasshook__', 'fromkeys', 'pipe_cloexec',
    )

    __slots__ = ('_data', '_inherited_attributes', 
        '_methods', '_object', '_prefix', '_readonly_properties', 
        '_readwrite_properties', '_special_methods')

    ### INITIALIZER ###

    def __init__(self, obj, prefix='abjad.tools.'):
        assert isinstance(obj, type)
        Documenter.__init__(self, obj, prefix)

        data = []
        inherited_attributes = []
        methods = []
        readonly_properties = []
        readwrite_properties = []
        special_methods = []

        attrs = inspect.classify_class_attrs(self._object)
        for attr in attrs:
            if attr.defining_class is object:
                continue
            if self._attribute_is_inherited(attr):
                inherited_attributes.append(attr)
            if attr.kind in ('class method', 'method'):
                if attr.name not in self._ignored_special_methods:
                    if attr.name.startswith('__'):
                        special_methods.append(attr)
                    elif not attr.name.startswith('_'):
                        methods.append(attr)
            elif attr.kind == 'property' and not attr.name.startswith('_'):
                if attr.object.fset is None:
                    readonly_properties.append(attr)
                else:
                    readwrite_properties.append(attr)
            elif attr.kind == 'data' and not attr.name.startswith('_'):
                data.append(attr)

        self._data = tuple(sorted(data))
        self._inherited_attributes = tuple(sorted(inherited_attributes))
        self._methods = tuple(sorted(methods))
        self._readonly_properties = tuple(sorted(readonly_properties))
        self._readwrite_properties = tuple(sorted(readwrite_properties))
        self._special_methods = tuple(sorted(special_methods))

    ### SPECIAL METHODS ###

    def __call__(self):
        '''Generate documentation.

        Returns string.
        '''
        from abjad.tools import documentationtools

        stripped_class_name = self._shrink_module_name(self.object.__module__)
        module_name = '{}.{}'.format(self.object.__module__, self.object.__name__)

        #result = []
        #result.extend(self._format_heading(stripped_class_name, '='))
        #result.extend(self._format_inheritance_diagram())
        #result.append('.. autoclass:: %s' % module_name)
        #result.append('   :noindex:')
        #result.append('')

        document = documentationtools.ReSTDocument()
        document.append(documentationtools.ReSTHeading(
            level=2,
            text=stripped_class_name,
            ))
        #document.append(documentationtools.ReSTInheritanceDiagram(
        #    argument=module_name,
        #    ))
        document.append(documentationtools.ReSTLineageDirective(
            argument=module_name,
            ))
        document.append(documentationtools.ReSTAutodocDirective(
            argument=module_name,
            directive='autoclass',
            options={'noindex': True},
            ))

        if self.readonly_properties:
            #result.extend(self._format_heading('Read-only Properties', '-'))
            #for attr in self.readonly_properties:
            #    result.extend(self._format_attribute(attr, 'attribute'))
            document.append(documentationtools.ReSTHeading(
                level=3,
                text='Read-only Properties',
                ))
            for attr in self.readonly_properties:
                autodoc = documentationtools.ReSTAutodocDirective(
                    argument='{}.{}'.format(module_name, attr.name),
                    directive='autoattribute',
                    options={'noindex': True},
                    )
                #autodoc.extend(self._format_inheritance_note(attr))
                document.append(autodoc)

        if self.readwrite_properties:
            #result.extend(self._format_heading('Read/write Properties', '-'))
            #for attr in self.readwrite_properties:
            #    result.extend(self._format_attribute(attr, 'attribute'))
            document.append(documentationtools.ReSTHeading(
                level=3,
                text='Read/write Properties',
                ))
            for attr in self.readwrite_properties:
                autodoc = documentationtools.ReSTAutodocDirective(
                    argument='{}.{}'.format(module_name, attr.name),
                    directive='autoattribute',
                    options={'noindex': True},
                    )
                #autodoc.extend(self._format_inheritance_note(attr))
                document.append(autodoc)

        if self.methods:
            #result.extend(self._format_heading('Methods', '-'))
            #for attr in self.methods:
            #    result.extend(self._format_attribute(attr, 'method'))
            document.append(documentationtools.ReSTHeading(
                level=3,
                text='Methods',
                ))
            for attr in self.methods:
                autodoc = documentationtools.ReSTAutodocDirective(
                    argument='{}.{}'.format(module_name, attr.name),
                    directive='automethod',
                    options={'noindex': True},
                    )
                #autodoc.extend(self._format_inheritance_note(attr))
                document.append(autodoc)

        if self.special_methods:
            #result.extend(self._format_heading('Special Methods', '-'))
            #for attr in self.special_methods:
            #    result.extend(self._format_attribute(attr, 'method'))
            document.append(documentationtools.ReSTHeading(
                level=3,
                text='Special Methods',
                ))
            for attr in self.special_methods:
                autodoc = documentationtools.ReSTAutodocDirective(
                    argument='{}.{}'.format(module_name, attr.name),
                    directive='automethod',
                    options={'noindex': True},
                    )
                #autodoc.extend(self._format_inheritance_note(attr))
                document.append(autodoc)

        #return '\n'.join(result)
        return document.rest_format

    ### PRIVATE METHODS ###

    def _attribute_is_inherited(self, attr):
        if attr.defining_class is not self._object:
            return True
        return False

    def _format_inheritance_note(self, attr):
        from abjad.tools import documentationtools
        if not self._attribute_is_inherited(attr):
            return []
        defining_module = '{}.{}'.format(attr.defining_class.__module__, attr.defining_class.__name__)
        if defining_module.startswith(('abjad.', 'experimental.')):
            stripped_class_name = self._shrink_module_name(defining_module)
            text='Inherited from :py:class:`{} <{}>`'.format(
                stripped_class_name, defining_module)
        else:
            text='Inherited from :py:class:`{}`'.format(defining_module)
        return [documentationtools.ReSTParagraph(text=text, wrap=False)]

    ### PUBLIC PROPERTIES ###

    @property
    def data(self):
        return self._data

    @property
    def inherited_attributes(self):
        return self._inherited_attributes

    @property
    def is_abstract(self):
        return inspect.isabstract(self.object)

    @property
    def methods(self):
        return self._methods

    @property
    def readonly_properties(self):
        return self._readonly_properties

    @property
    def readwrite_properties(self):
        return self._readwrite_properties

    @property
    def special_methods(self):
        return self._special_methods




from abjad.tools import componenttools
from abjad.tools import iterationtools
from abjad.tools import leaftools
from abjad.tools import markuptools
from abjad.tools import notetools


def label_leaves_in_expr_with_melodic_chromatic_interval_classes(expr, markup_direction=Up):
    r""".. versionadded:: 2.0

    Label leaves in `expr` with melodic chromatic interval classes::

        >>> notes = notetools.make_notes([0, 25, 11, -4, -14, -13, 9, 10, 6, 5], [Duration(1, 8)])
        >>> staff = Staff(notes)
        >>> labeltools.label_leaves_in_expr_with_melodic_chromatic_interval_classes(staff)

    ::

        >>> f(staff)
        \new Staff {
            c'8 ^ \markup { +1 }
            cs'''8 ^ \markup { -2 }
            b'8 ^ \markup { -2 }
            af8 ^ \markup { -10 }
            bf,8 ^ \markup { +1 }
            b,8 ^ \markup { +10 }
            a'8 ^ \markup { +1 }
            bf'8 ^ \markup { -4 }
            fs'8 ^ \markup { -1 }
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return none.
    """

    for note in iterationtools.iterate_notes_in_expr(expr):
        thread_iterator = iterationtools.iterate_thread_from_component(note, leaftools.Leaf)
        try:
            thread_iterator.next()
            next_leaf = thread_iterator.next()
            if isinstance(next_leaf, notetools.Note):
                mdi = note.written_pitch - next_leaf.written_pitch
                mci = mdi.melodic_chromatic_interval
                mcic = mci.melodic_chromatic_interval_class
                markuptools.Markup(mcic, markup_direction)(note)
        except StopIteration:
            pass



from abjad.tools import contexttools
from abjad.tools import durationtools
from abjad.tools.quantizationtools.QSchemaItem import QSchemaItem


class BeatwiseQSchemaItem(QSchemaItem):
    '''`BeatwiseQSchemaItem` represents a change of state in the timeline of an unmetered
    quantization process.

    ::

        >>> from abjad.tools import quantizationtools
        >>> quantizationtools.BeatwiseQSchemaItem()
        BeatwiseQSchemaItem()

    Define a change in tempo:

    ::

        >>> quantizationtools.BeatwiseQSchemaItem(tempo=((1, 4), 60))
        BeatwiseQSchemaItem(
            tempo=contexttools.TempoMark(
                    durationtools.Duration(1, 4),
                    60
                    ),
            )
           
    Define a change in beatspan:

    ::

        >>> quantizationtools.BeatwiseQSchemaItem(beatspan=(1, 8))
        BeatwiseQSchemaItem(
            beatspan=durationtools.Duration(1, 8),
            )

    `BeatwiseQSchemaItem` is immutable.

    Return `BeatwiseQSchemaItem` instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ()
    _fields = ('beatspan', 'search_tree', 'tempo')

    ### INITIALIZER ###

    def __init__(self, beatspan=None, search_tree=None, tempo=None):
        pass

    def __new__(klass, beatspan=None, search_tree=None, tempo=None):
        from abjad.tools import quantizationtools

        if beatspan is not None:
            beatspan = durationtools.Duration(beatspan)
            assert 0 < beatspan

        if search_tree is not None:
            assert isinstance(search_tree, quantizationtools.SearchTree)

        if tempo is not None:
            tempo = contexttools.TempoMark(tempo)
            assert not tempo.is_imprecise

        return tuple.__new__(klass, (beatspan, search_tree, tempo))

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def beatspan(self):
        '''The optionally defined beatspan duration.

        Return `Duration` or `None`.
        '''
        return self[0]

    @property
    def search_tree(self):
        '''The optionally defined search tree.

        Return `OldSearchTree` or `None`.
        '''
        return self[1]

    @property
    def tempo(self):
        '''The optionally defined `TempoMark`.

        Return `TempoMark` or `None`.
        '''
        return self[2]



from abjad.tools import schemetools
from abjad.tools.markuptools.MarkupCommand import MarkupCommand


class MusicGlyph(MarkupCommand):
    '''Abjad model of a LilyPond \musicglyph command:

    ::

        >>> markuptools.MusicGlyph('accidentals.sharp')
        MusicGlyph('accidentals.sharp')
        >>> print _
        \musicglyph #"accidentals.sharp"

    Return `MusicGlyph` instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, glyph_name):
        from abjad.ly import music_glyphs
        assert glyph_name in music_glyphs, 'Not a valid LilyPond glyph name.'
        glyph_scheme = schemetools.Scheme(glyph_name, force_quotes=True)
        MarkupCommand.__init__(self, 'musicglyph', glyph_scheme)

    ### SPECIAL METHODS ### 

    def __repr__(self):
        return '{}({!r})'.format(self._class_name, self.args[0]._value)



from abjad import *


def test_tietools_iterate_tie_chains_in_expr_01():
    '''Yield successive tie chains.
    '''

    t = Staff(notetools.make_repeated_notes(4))
    tietools.TieSpanner(t[:2])
    tietools.TieSpanner(t[2:])

    r'''
    \new Staff {
        c'8 ~
        c'8
        c'8 ~
        c'8
    }
    '''

    chains = list(tietools.iterate_tie_chains_in_expr(t, reverse=True))

    assert chains[0] == tietools.TieChain((t[2], t[3]))
    assert chains[1] == tietools.TieChain((t[0], t[1]))


def test_tietools_iterate_tie_chains_in_expr_02():
    '''Yield successive tie chains.
    '''

    t = Staff(notetools.make_repeated_notes(4))

    r'''
    \new Staff {
        c'8
        c'8
        c'8
        c'8
    }
    '''

    chains = list(tietools.iterate_tie_chains_in_expr(t, reverse=True))

    assert chains[0] == tietools.TieChain(t[3])
    assert chains[1] == tietools.TieChain(t[2])
    assert chains[2] == tietools.TieChain(t[1])
    assert chains[3] == tietools.TieChain(t[0])


def test_tietools_iterate_tie_chains_in_expr_03():
    '''Yield successive tie chains.
    '''

    t = Staff(notetools.make_repeated_notes(4))
    tietools.TieSpanner(t[:2])
    tietools.TieSpanner(t[2:])

    r'''
    \new Staff {
        c'8 ~
        c'8
        c'8 ~
        c'8
    }
    '''

    chains = list(tietools.iterate_tie_chains_in_expr(t))

    assert chains[0] == tietools.TieChain((t[0], t[1]))
    assert chains[1] == tietools.TieChain((t[2], t[3]))


def test_tietools_iterate_tie_chains_in_expr_04():
    '''Yield successive tie chains.
    '''

    t = Staff(notetools.make_repeated_notes(4))

    r'''
    \new Staff {
        c'8
        c'8
        c'8
        c'8
    }
    '''

    chains = list(tietools.iterate_tie_chains_in_expr(t))

    assert chains[0] == tietools.TieChain(t[0])
    assert chains[1] == tietools.TieChain(t[1])
    assert chains[2] == tietools.TieChain(t[2])
    assert chains[3] == tietools.TieChain(t[3])



from abjad.tools import componenttools
from abjad.tools import containertools


def get_measure_that_starts_with_container(container):
    '''.. versionadded:: 2.11

    Get measure that starts with `container`.

    Return measure or none.
    '''
    from abjad.tools import measuretools

    if isinstance(container, containertools.Container):
        contents = componenttools.get_improper_descendents_of_component_that_start_with_component(container)
        contents = [x for x in contents if isinstance(x, measuretools.Measure)]
        if contents:
            return contents[0]
        raise MissingMeasureError



import argparse
import multiprocessing
import os
import pytest
from abjad.tools import iotools
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript


class PyTestScript(DirectoryScript):
    '''Run `py.test` on various Abjad paths:

    ::

        bash$ ajv test -h
        usage: py-test [-h] [--version] [-p] [-x] [-A | -D | -M | -X]

        Run "py.test" on various Abjad paths.

        optional arguments:
          -h, --help          show this help message and exit
          --version           show program's version number and exit
          -p, --parallel      run py.test with multiprocessing
          -x, --exitfirst     stop on first failure
          -A, --all           test all directories, including demos
          -D, --demos         test demos directory
          -M, --mainline      test mainline tools directory
          -X, --experimental  test experimental directory

    Return `PyTestScript` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'test'

    @property
    def long_description(self):
        return None

    @property
    def scripting_group(self):
        return None

    @property
    def short_description(self):
        return 'Run "py.test" on various Abjad paths.'

    @property
    def version(self):
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):

        parallel = ''
        if args.parallel:
            parallel = '-n {}'.format(multiprocessing.cpu_count())
        
        exitfirst = ''
        if args.exitfirst:
           exitfirst = '-x' 

        report = ''
        if args.report:
            report = '-r {}'.format(args.report)

        print 'TESTING:'
        for path in args.path:
            print '\t{}'.format(path)
        print ''

        path = ' '.join(args.path)
        command = '{} {} {} {}'.format(parallel, exitfirst, report, path)

        return pytest.main(command.split())

    def setup_argument_parser(self, parser):

        from abjad import ABJCFG

        parser.add_argument('-p', '--parallel',
            action='store_true',
            dest='parallel',
            help='run py.test with multiprocessing',
            )

        parser.add_argument('-r', '--report',
            action='store',
            dest='report',
            help='show extra test summary info as specified by chars ' + \
                '(f)ailed, (E)error, (s)skipped, (x)failed, (X)passed.',
            metavar='chars',
            )

        parser.add_argument('-x', '--exitfirst',
            action='store_true',
            dest='exitfirst',
            help='stop on first failure',
            )

        group = parser.add_mutually_exclusive_group()

        group.add_argument('-A', '--all',
            action='store_const',
            const=[ABJCFG.ABJAD_ROOT_PATH],
            dest='path',
            help='test all directories, including demos',
            )

        group.add_argument('-D', '--demos',
            action='store_const',
            const=[os.path.join(ABJCFG.ABJAD_PATH, 'demos')],
            dest='path',
            help='test demos directory',
            )

        group.add_argument('-M', '--mainline',
            action='store_const',
            const=[os.path.join(ABJCFG.ABJAD_PATH, 'tools')],
            dest='path',
            help='test mainline tools directory',
            )

        group.add_argument('-X', '--experimental',
            action='store_const',
            const=[ABJCFG.ABJAD_EXPERIMENTAL_PATH],
            dest='path',
            help='test experimental directory',
            )

        parser.set_defaults(path=[
            os.path.join(ABJCFG.ABJAD_PATH, 'tools'),
            ABJCFG.ABJAD_EXPERIMENTAL_PATH
            ])




import abc

from abjad.tools.pitchtools.NumberedObject import NumberedObject
from abjad.tools.pitchtools.PitchObject import PitchObject


class NumberedPitchObject(PitchObject, NumberedObject):
    '''.. versionadded:: 2.0

    Numbered pitch base class from which concrete classes inherit.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    __slots__ = ()

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        pass



from abjad import *


def test_Markup___init___01():
    '''Init markup with string.
    '''

    markup = markuptools.Markup('foo')
    assert str(markup) == '\\markup { foo }'


def test_Markup___init___02():
    '''Init markup with other markup instance.
    '''

    markup_1 = markuptools.Markup('foo')
    markup_2 = markuptools.Markup(markup_1)

    assert str(markup_1) == '\\markup { foo }'
    assert str(markup_2) == '\\markup { foo }'


def test_Markup___init___03():
    '''Init markup with nonstring and nonmarkup instance.
    '''

    markup = markuptools.Markup(27)
    assert str(markup) == '\\markup { 27 }'



def test_Markup___init___04():
    '''Init markup from MarkupCommand.
    '''

    command = markuptools.MarkupCommand('flat')
    markup = markuptools.Markup(command)
    assert str(markup) == '\\markup { \\flat }'


def test_Markup___init___05():
    '''Init markup from sequence of strings or MarkupCommands.
    '''

    command_1 = markuptools.MarkupCommand('flat')
    command_2 = markuptools.MarkupCommand('sharp')
    markup = markuptools.Markup(['X', command_1, 'Y', command_2, 'Z'])
    assert str(markup) == '\\markup { X \\flat Y \\sharp Z }'


def test_Markup___init___06():
    '''Preserve all keywords when initializing from other markup instance.
    '''

    markup_1 = markuptools.Markup(
        'foo contents string', direction=Up, markup_name='foo')
    markup_2 = markuptools.Markup(markup_1)

    assert markup_1 is not markup_2
    assert markup_1 == markup_2
    assert repr(markup_1) == repr(markup_2)
    assert markup_1._storage_format == markup_2._storage_format


def test_Markup___init___07():
    '''Preserve keywords when initializing from other markup instance
    but also overwrite keywords specified anew.
    '''

    markup_1 = markuptools.Markup(
        'foo contents string', direction=Up, markup_name='foo')
    markup_2 = markuptools.Markup(markup_1, direction=Down)

    assert markup_1 is not markup_2
    assert markup_2.contents == ('foo contents string',)
    assert markup_2.direction is Down
    assert markup_2.markup_name == 'foo'



import abc
import os
from abjad.tools import abctools


class OutputFormat(abctools.AbjadObject):

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_code_block_closing', '_code_block_opening', '_code_indent', '_image_block', '_image_format')

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, code_block_opening, code_block_closing, code_indent, image_block, image_format):
        self._code_block_opening = code_block_opening
        self._code_block_closing = code_block_closing
        self._code_indent = code_indent
        self._image_block = image_block
        self._image_format = image_format

    ### SPECIAL METHODS ###

    def __call__(self, code_block, image_dict):

        reformatted = []
        for result in code_block.processed_results:
            if isinstance(result, tuple):
                reformatted.append(
                    self.code_block_opening +
                    '\n'.join([(' ' * self.code_indent) + x for x in result]) +
                    self.code_block_closing)

            elif isinstance(result, dict):
                file_name = result['file_name']
                image_count = result['image_count']
                image_prefix = result['image_prefix']
                page_range = result['page_range']

                if page_range is None:
                    image_file_names = [v for k, v in sorted(image_dict[image_count].items())]
                    for image_file_name in image_file_names:
                        image_file_name = image_file_name.rpartition('.')[0]
                        reformatted.append(self.image_block.format(image_file_name))

                else:
                    for page_number, image_file_name in sorted(image_dict[image_count].items()):
                        if page_number in page_range:
                            image_file_name = image_file_name.rpartition('.')[0]
                            reformatted.append(self.image_block.format(image_file_name))
                        else:
                            os.remove(image_file_name)
                            
        return tuple(reformatted)

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def code_block_closing(self):
        return self._code_block_closing

    @property
    def code_block_opening(self):
        return self._code_block_opening

    @property
    def code_indent(self):
        return self._code_indent

    @property
    def image_block(self):
        return self._image_block

    @property
    def image_format(self):
        return self._image_format



from abjad.tools import configurationtools
import os


def save_last_ly_as(file_name):
    r'''.. versionadded:: 2.0

    Save last ly file as `file_name`::

        >>> iotools.save_last_ly_as('/project/output/example-1.ly') # doctest: +SKIP

    Return none.
    '''
    from abjad import ABJCFG
    from abjad.tools import iotools

    ABJADOUTPUT = ABJCFG['abjad_output']
    last_ly = iotools.get_last_output_file_name()
    last_ly_full_name = os.path.join(ABJADOUTPUT, last_ly)
    old = open(last_ly_full_name, 'r')
    new = open(file_name, 'w')
    new.write(''.join(old.readlines()))
    old.close()
    new.close()



from abjad.tools import schemetools
from abjad.tools.spannertools.Spanner import Spanner


class StaffLinesSpanner(Spanner):
    r'''Abjad staff lines spanner::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> spannertools.StaffLinesSpanner(staff[:2], 1)
        StaffLinesSpanner(c'8, d'8)

    ::

        >>> f(staff)
        \new Staff {
            \stopStaff
            \override Staff.StaffSymbol #'line-count = #1
            \startStaff
            c'8
            d'8
            \stopStaff
            \revert Staff.StaffSymbol #'line-count
            \startStaff
            e'8
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Staff lines spanner handles changing either the line-count
    or the line-positions property of the StaffSymbol grob,
    as well as automatically stopping and restarting the staff
    so that the change may take place.

    Return staff lines spanner.
    '''

    ### INITIALIZER ###

    def __init__(self, components=None, lines=5):
        Spanner.__init__(self, components)
        self.lines = lines

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new.lines = self.lines

    def _format_after_leaf(self, leaf):
        result = []
        if self._is_my_last_leaf(leaf):
            result.append(r'\stopStaff')
            if isinstance(self.lines, int):
                result.append(r"\revert Staff.StaffSymbol #'line-count")
            else:
                result.append(r"\revert Staff.StaffSymbol #'line-positions")
            result.append(r'\startStaff')
        return result

    def _format_before_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r'\stopStaff')
            if isinstance(self.lines, int):
                result.append(r"\override Staff.StaffSymbol #'line-count = #%s" % \
                    self.lines)
            else:
                result.append(r"\override Staff.StaffSymbol #'line-positions = %s" % \
                    schemetools.SchemeVector(*self.lines).lilypond_format)
            result.append(r'\startStaff')
        return result

    ### READ / WRITE PUBLIC PROPERTIES ###

    @apply
    def lines():
        def fget(self):
            r'''Get staff lines spanner line count::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> spanner = spannertools.StaffLinesSpanner(staff[:2], 1)
                >>> spanner.lines
                1

            Set staff lines spanner line count::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> spanner = spannertools.StaffLinesSpanner(staff[:2], 1)
                >>> spanner.lines = 2
                >>> spanner.lines
                2

            Set integer.
            '''
            return self._lines
        def fset(self, arg):
            if isinstance(arg, int) and 0 < arg:
                self._lines = arg
            elif isinstance(arg, (tuple, list)) \
                and all([isinstance(x, (int, float)) for x in arg]):
                self._lines = tuple(arg)
            else:
                raise ValueError('StaffLinesSpanner requires either an int, '
                    'or a list/tuple of ints and/or floats.')
        return property(**locals())



from abjad.tools import componenttools
from abjad.tools import durationtools
from abjad.tools import pitchtools


# TODO: This be replaced in favor of leaftools.split_leaf_at_offsets().
#       The precondition is that leaftools.split_leaf_at_offsets() must be
#       extended to handle graces.
#       Also important to migrate over the (large-ish) set of tests for this i
#       function.
def split_leaf_at_offset(leaf, offset, fracture_spanners=False,
    tie_split_notes=True, tie_split_rests=False):
    r'''Split `leaf` at `offset`.

    Example 1. Split note at assignable offset. Two notes result. Do not tie notes::

        >>> staff = Staff(r"abj: | 2/8 c'8 ( d'8 || 2/8 e'8 f'8 ) |")
        >>> beamtools.apply_beam_spanners_to_measures_in_expr(staff)
        [BeamSpanner(|2/8(2)|), BeamSpanner(|2/8(2)|)]
        >>> contexttools.DynamicMark('f')(staff.leaves[0])
        DynamicMark('f')(c'8)
        >>> marktools.Articulation('accent')(staff.leaves[0])
        Articulation('accent')(c'8)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8 -\accent \f [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }

    ::

        >>> leaftools.split_leaf_at_offset(staff.leaves[0], (1, 32),
        ...     tie_split_notes=False)
        ([Note("c'32")], [Note("c'16.")])

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'32 -\accent \f [ (
                c'16.
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }

    Example 2. Handle grace and after grace containers correctly.

        >>> staff = Staff(r"abj: | 2/8 c'8 ( d'8 || 2/8 e'8 f'8 ) |")
        >>> beamtools.apply_beam_spanners_to_measures_in_expr(staff)
        [BeamSpanner(|2/8(2)|), BeamSpanner(|2/8(2)|)]
        >>> gracetools.GraceContainer("cs'16")(staff.leaves[0])
        Note("c'8")
        >>> gracetools.GraceContainer("ds'16", kind='after')(staff.leaves[0])
        Note("c'8")

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                \grace {
                    cs'16
                }
                \afterGrace
                c'8 [ (
                {
                    ds'16
                }
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }

    ::

        >>> leaftools.split_leaf_at_offset(staff.leaves[0], (1, 32),
        ...     tie_split_notes=False)
        ([Note("c'32")], [Note("c'16.")])

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                \grace {
                    cs'16
                }
                c'32 [ (
                \afterGrace
                c'16.
                {
                    ds'16
                }
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }

    Return pair.
    '''
    from abjad.tools import contexttools
    from abjad.tools import gracetools
    from abjad.tools import leaftools
    from abjad.tools import marktools
    from abjad.tools import spannertools
    from abjad.tools import tietools

    # check input
    assert isinstance(leaf, leaftools.Leaf)
    offset = durationtools.Offset(offset)

    # calculate durations
    leaf_multiplied_duration = leaf.multiplied_duration
    preprolated_duration = offset / leaf.prolation

    # handle boundary cases
    if preprolated_duration <= 0:
        return ([], [leaf])
    if leaf_multiplied_duration <= preprolated_duration:
        return ([leaf], [])

    # create new leaf
    new_leaf = componenttools.copy_components_and_remove_spanners([leaf])[0]
    componenttools.extend_in_parent_of_component(leaf, [new_leaf], grow_spanners=True)

    # adjust leaf
    gracetools.detach_grace_containers_attached_to_leaf(leaf, kind='after')

    # adjust new leaf
    gracetools.detach_grace_containers_attached_to_leaf(new_leaf, kind='grace')
    marktools.detach_marks_attached_to_component(new_leaf) 
    contexttools.detach_context_marks_attached_to_component(new_leaf)

    left_leaf_list = leaftools.set_preprolated_leaf_duration(leaf, preprolated_duration)
    right_preprolated_duration = leaf_multiplied_duration - preprolated_duration
    right_leaf_list = leaftools.set_preprolated_leaf_duration(new_leaf,
        right_preprolated_duration)

    leaf_left_of_split = left_leaf_list[-1]
    leaf_right_of_split = right_leaf_list[0]

    if fracture_spanners:
        spannertools.fracture_spanners_attached_to_component(leaf_left_of_split, 
        direction=Right)

    # tie split notes, rests and chords as specified
    if  (pitchtools.is_pitch_carrier(leaf) and tie_split_notes) or \
        (not pitchtools.is_pitch_carrier(leaf) and tie_split_rests):
        tietools.apply_tie_spanner_to_leaf_pair(leaf_left_of_split, leaf_right_of_split)

    return left_leaf_list, right_leaf_list

    # TODO: make this substitution work
#    return leaftools.split_leaf_at_offsets(leaf, [offset], cyclic=False, 
#        fracture_spanners=fracture_spanners, tie_split_notes=tie_split_notes,
#        tie_split_rests=tie_split_rests)



class SpannerPopulationError(Exception):
    '''Spanner contents incorrect.

    Spanner may be missing component it is assumed to have.

    Spanner may have a component it is assumed not to have.
    '''
    pass



from abjad.tools import componenttools


def get_nth_leaf_in_thread_from_leaf(leaf, n=0):
    r'''.. versionadded:: 2.0

    Get `n` th leaf in thread from `leaf`::

        >>> staff = Staff(2 * Voice("c'8 d'8 e'8 f'8"))
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)

    ::

        >>> f(staff)
        \new Staff {
            \new Voice {
                c'8
                d'8
                e'8
                f'8
            }
            \new Voice {
                g'8
                a'8
                b'8
                c''8
            }
        }

    ::

        >>> for n in range(8):
        ...     print n, leaftools.get_nth_leaf_in_thread_from_leaf(staff[0][0], n)
        ...
        0 c'8
        1 d'8
        2 e'8
        3 f'8
        4 None
        5 None
        6 None
        7 None

    Return leaf or none.
    '''
    from abjad.tools import leaftools

    if not isinstance(leaf, leaftools.Leaf):
        return None

    def _next(component):
        new_component = componenttools.get_nth_component_in_time_order_from_component(component, 1)
        if new_component is None:
            return
        candidates = componenttools.get_improper_descendents_of_component_that_start_with_component(
            new_component)
        candidates = [x for x in candidates if isinstance(x, leaftools.Leaf)]
        for candidate in candidates:
            if componenttools.all_are_components_in_same_thread([component, candidate]):
                return candidate

    def _prev(component):
        new_component = componenttools.get_nth_component_in_time_order_from_component(component, -1)
        if new_component is None:
            return
        candidates = componenttools.get_improper_descendents_of_component_that_stop_with_component(
            new_component)
        candidates = [x for x in candidates if isinstance(x, leaftools.Leaf)]
        for candidate in candidates:
            if componenttools.all_are_components_in_same_thread([component, candidate]):
                return candidate

    current_leaf = leaf

    if n < 0:
        for i in range(abs(n)):
            current_leaf = _prev(current_leaf)
            if current_leaf is None:
                break
    elif n == 0:
        pass
    else:
        for i in range(n):
            current_leaf = _next(current_leaf)
            if current_leaf is None:
                break

    return current_leaf



from abjad.tools import scoretemplatetools


def test_StringQuartetScoreTemplate___call___01():

    template = scoretemplatetools.StringQuartetScoreTemplate()
    score = template()

    r'''
    \context Score = "String Quartet Score" <<
        \context StaffGroup = "String Quartet Staff Group" <<
            \context Staff = "First Violin Staff" {
                \clef "treble"
                \set Staff.instrumentName = \markup { Violin }
                \set Staff.shortInstrumentName = \markup { Vn. }
                \context Voice = "First Violin Voice" {
                }
            }
            \context Staff = "Second Violin Staff" {
                \clef "treble"
                \set Staff.instrumentName = \markup { Violin }
                \set Staff.shortInstrumentName = \markup { Vn. }
                \context Voice = "Second Violin Voice" {
                }
            }
            \context Staff = "Viola Staff" {
                \clef "alto"
                \set Staff.instrumentName = \markup { Viola }
                \set Staff.shortInstrumentName = \markup { Va. }
                \context Voice = "Viola Voice" {
                }
            }
            \context Staff = "Cello Staff" {
                \clef "bass"
                \set Staff.instrumentName = \markup { Cello }
                \set Staff.shortInstrumentName = \markup { Vc. }
                \context Voice = "Cello Voice" {
                }
            }
        >>
    >>
    '''

    assert score.lilypond_format == '\\context Score = "String Quartet Score" <<\n\t\\context StaffGroup = "String Quartet Staff Group" <<\n\t\t\\context Staff = "First Violin Staff" {\n\t\t\t\\clef "treble"\n\t\t\t\\set Staff.instrumentName = \\markup { Violin }\n\t\t\t\\set Staff.shortInstrumentName = \\markup { Vn. }\n\t\t\t\\context Voice = "First Violin Voice" {\n\t\t\t}\n\t\t}\n\t\t\\context Staff = "Second Violin Staff" {\n\t\t\t\\clef "treble"\n\t\t\t\\set Staff.instrumentName = \\markup { Violin }\n\t\t\t\\set Staff.shortInstrumentName = \\markup { Vn. }\n\t\t\t\\context Voice = "Second Violin Voice" {\n\t\t\t}\n\t\t}\n\t\t\\context Staff = "Viola Staff" {\n\t\t\t\\clef "alto"\n\t\t\t\\set Staff.instrumentName = \\markup { Viola }\n\t\t\t\\set Staff.shortInstrumentName = \\markup { Va. }\n\t\t\t\\context Voice = "Viola Voice" {\n\t\t\t}\n\t\t}\n\t\t\\context Staff = "Cello Staff" {\n\t\t\t\\clef "bass"\n\t\t\t\\set Staff.instrumentName = \\markup { Cello }\n\t\t\t\\set Staff.shortInstrumentName = \\markup { Vc. }\n\t\t\t\\context Voice = "Cello Voice" {\n\t\t\t}\n\t\t}\n\t>>\n>>'



from abjad.tools import componenttools
from abjad.tools import durationtools
from abjad.tools import mathtools


def divide_leaf_meiotically(leaf, n=2):
    r'''.. versionadded:: 1.1

    Divide `leaf` meiotically `n` times::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> beamtools.BeamSpanner(staff.leaves)
        BeamSpanner(c'8, d'8, e'8, f'8)
        >>> f(staff)
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8 ]
        }

    ::

        >>> leaftools.divide_leaf_meiotically(staff[0], n=4)

    ::

        >>> f(staff)
        \new Staff {
            c'32 [
            c'32
            c'32
            c'32
            d'8
            e'8
            f'8 ]
        }


    Replace `leaf` with `n` new leaves.

    Preserve parentage and spanners.

    Allow divisions into only ``1, 2, 4, 8, 16, ...`` and other
    nonnegative integer powers of ``2``.

    Produce only leaves and never tuplets or other containers.

    Return none.
    '''
    from abjad.tools import leaftools

    # TODO: find a way to optimize this; either reimplement
    # componenttools.extend_in_parent_of_component()
    # or come up with something else.

    assert isinstance(leaf, leaftools.Leaf)
    assert mathtools.is_nonnegative_integer_power_of_two(n)
    assert 0 < n

    new_leaves = componenttools.copy_components_and_remove_spanners([leaf], n - 1)
    componenttools.extend_in_parent_of_component(leaf, new_leaves, grow_spanners=True)
    adjustment_multiplier = durationtools.Duration(1, n)
    leaf.written_duration *= adjustment_multiplier
    for new_leaf in new_leaves:
        new_leaf.written_duration *= adjustment_multiplier



from abjad.tools import durationtools
from abjad.tools.schemetools.Scheme import Scheme


class SchemeMoment(Scheme):
    '''Abjad model of LilyPond moment::

        >>> schemetools.SchemeMoment(1, 68)
        SchemeMoment((1, 68))

    Initialize scheme moments with a single fraction, two integers or another scheme moment.

    Scheme moments are immutable.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, *args, **kwargs):
        if len(args) == 1 and durationtools.Duration.is_token(args[0]):
            args = durationtools.Duration(args[0])
        elif len(args) == 1 and isinstance(args[0], type(self)):
            args = args[0].duration
        elif len(args) == 2 and isinstance(args[0], int) and isinstance(args[1], int):
            args = durationtools.Duration(args)
        else:
            raise TypeError('can not intialize scheme moment from "{}".'.format(args))
        Scheme.__init__(self, args, **kwargs)

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        if isinstance(arg, type(self)):
            if self._value == arg._value:
                return True
        return False

    def __ge__(self, arg):
        if isinstance(arg, type(self)):
            if self._value >= arg._value:
                return True
        return False

    def __getnewargs__(self):
        return (self._value,)

    def __gt__(self, arg):
        if isinstance(arg, type(self)):
            if self._value > arg._value:
                return True
        return False

    def __le__(self, arg):
        if isinstance(arg, type(self)):
            if self._value <= arg._value:
                return True
        return False

    def __lt__(self, arg):
        if isinstance(arg, type(self)):
            if self._value < arg._value:
                return True
        return False

    def __ne__(self, arg):
        return not self == arg

    def __repr__(self):
        return '{}(({}, {}))'.format(
            type(self).__name__, self._value.numerator, self._value.denominator)

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _formatted_value(self):
        numerator, denominator = self._value.numerator, self._value.denominator
        return '(ly:make-moment %s %s)' % (numerator, denominator)

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def duration(self):
        '''Duration of scheme moment::

            >>> scheme_moment = schemetools.SchemeMoment(1, 68)
            >>> scheme_moment.duration
            Duration(1, 68)

        Return duration.
        '''
        return self._value



def list_instruments(klasses=None):
    r'''.. versionadded:: 2.5

    List instruments in ``instrumenttools`` module::

        >>> for instrument in instrumenttools.list_instruments()[:5]:
        ...     instrument
        ... 
        <class 'abjad.tools.instrumenttools.Accordion.Accordion.Accordion'>
        <class 'abjad.tools.instrumenttools.AltoFlute.AltoFlute.AltoFlute'>
        <class 'abjad.tools.instrumenttools.AltoSaxophone.AltoSaxophone.AltoSaxophone'>
        <class 'abjad.tools.instrumenttools.AltoTrombone.AltoTrombone.AltoTrombone'>
        <class 'abjad.tools.instrumenttools.BFlatClarinet.BFlatClarinet.BFlatClarinet'>

    Return list.
    '''
    from abjad.tools import instrumenttools
    from abjad.tools.instrumenttools._Instrument import _Instrument

    if klasses is None:
        klasses = (_Instrument, )

    instruments = []
    for value in instrumenttools.__dict__.itervalues():
        try:
            if issubclass(value, klasses):
                instruments.append(value)
        except TypeError:
            pass
    instruments.sort(lambda x, y: cmp(x.__name__, y.__name__))

    return instruments



from abjad.tools import containertools
from abjad.tools import durationtools
from abjad.tools import mathtools


def move_full_measure_tuplet_prolation_to_measure_time_signature(expr):
    r'''.. versionadded:: 1.1

    Move prolation of full-measure tuplet to time signature of measure.

    Measures usually become non-power-of-two as as result::

        >>> t = Measure((2, 8), [tuplettools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")])
        >>> measuretools.move_full_measure_tuplet_prolation_to_measure_time_signature(t)

    ::

        >>> f(t)
        {
            \time 3/12
            \scaleDurations #'(2 . 3) {
                c'8
                d'8
                e'8
            }
        }

    Return none.

    .. versionchanged:: 2.0
        renamed ``measuretools.subsume()`` to
        ``measuretools.move_full_measure_tuplet_prolation_to_measure_time_signature()``.
    '''
    from abjad.tools import componenttools
    from abjad.tools import contexttools
    from abjad.tools import iterationtools
    from abjad.tools import tuplettools

    for measure in iterationtools.iterate_measures_in_expr(expr):
        if len(measure) == 1:
            if isinstance(measure[0], tuplettools.Tuplet):
                tuplet = measure[0]
                tuplet_multiplier = tuplet.multiplier
                tuplet_denominator = tuplet_multiplier.denominator
                reduced_denominator = mathtools.remove_powers_of_two(tuplet_denominator)
                time_signature = contexttools.get_effective_time_signature(measure)
                time_signature_rational = durationtools.Duration(
                    time_signature.numerator, time_signature.denominator)
                numerator = time_signature_rational.numerator * reduced_denominator
                denominator = time_signature_rational.denominator * reduced_denominator
                time_signature = contexttools.TimeSignatureMark((numerator, denominator))
                contexttools.detach_time_signature_marks_attached_to_component(measure)
                time_signature.attach(measure)
                time_signature_multiplier = \
                    contexttools.get_effective_time_signature(measure).implied_prolation
                written_adjustment = tuplet_multiplier / time_signature_multiplier
                componenttools.move_parentage_and_spanners_from_components_to_components(
                    [tuplet], tuplet[:])
                containertools.scale_contents_of_container(measure, written_adjustment)



from abjad.tools import iterationtools


def color_leaves_in_expr(expr, color):
    r""".. versionadded:: 2.0

    Color leaves in `expr`::

        >>> staff = Staff("cs'8. [ r8. s8. <c' cs' a'>8. ]")
        
    ::

        >>> f(staff)
        \new Staff {
            cs'8. [
            r8.
            s8.
            <c' cs' a'>8. ]
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> labeltools.color_leaves_in_expr(staff, 'red')

    ::

        >>> f(staff)
        \new Staff {
            \once \override Accidental #'color = #red
            \once \override Dots #'color = #red
            \once \override NoteHead #'color = #red
            cs'8. [
            \once \override Dots #'color = #red
            \once \override Rest #'color = #red
            r8.
            s8.
            \once \override Accidental #'color = #red
            \once \override Dots #'color = #red
            \once \override NoteHead #'color = #red
            <c' cs' a'>8. ]
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return none.
    """
    from abjad.tools import labeltools

    for leaf in iterationtools.iterate_leaves_in_expr(expr):
        labeltools.color_leaf(leaf, color)



def is_component_with_dynamic_mark_attached(expr):
    r'''.. versionadded:: 2.3

    True when `expr` is a component and has a dynamic mark attached::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> contexttools.DynamicMark('p')(staff[0])
        DynamicMark('p')(c'8)

    ::

        >>> f(staff)
        \new Staff {
            c'8 \p
            d'8
            e'8
            f'8
        }

    ::

        >>> contexttools.is_component_with_dynamic_mark_attached(staff[0])
        True

    Otherwise false::

        >>> contexttools.is_component_with_dynamic_mark_attached(staff)
        False

    Return boolean.
    '''
    from abjad.tools import contexttools

    return contexttools.is_component_with_context_mark_attached(expr, (contexttools.DynamicMark,))



def is_integer_pair(expr):
    r'''.. versionadded:: 2.9

    True when `expr` is an integer tuple of length ``2``::

        >>> sequencetools.is_integer_pair((19, 20))
        True

    Otherwise false::

        >>> sequencetools.is_integer_pair(('some', 'text'))
        False

    Return boolean.
    '''

    return isinstance(expr, tuple) and len(expr) == 2 and all([isinstance(x, int) for x in expr])



from abjad.tools import contexttools
from abjad.tools import durationtools


def scale_measure_denominator_and_adjust_measure_contents(measure, factor):
    r'''.. versionadded:: 1.1

    Change power-of-two `measure` to non-power-of-two measure with new denominator `factor`::

        >>> measure = Measure((2, 8), "c'8 d'8")
        >>> beamtools.BeamSpanner(measure.leaves)
        BeamSpanner(c'8, d'8)

    ::

        >>> f(measure)
        {
            \time 2/8
            c'8 [
            d'8 ]
        }

    ::

        >>> measuretools.scale_measure_denominator_and_adjust_measure_contents(measure, 3)
        Measure(3/12, [c'8., d'8.])

    ::

        >>> f(measure)
        {
            \time 3/12
            \scaleDurations #'(2 . 3) {
                c'8. [
                d'8. ]
            }
        }


    Treat new denominator `factor` like clever form of ``1``:
    ``3/3`` or ``5/5`` or ``7/7``, etc.

    Preserve `measure` prolated duration.

    Derive new `measure` multiplier.

    Scale `measure` contents.

    Pick best new time signature.
    '''
    from abjad.tools import measuretools
    from abjad.tools import timesignaturetools

    # save old time signature duration
    old_time_signature_duration = contexttools.get_effective_time_signature(measure).duration

    # find new time signature
    new_time_signature = timesignaturetools.duration_and_possible_denominators_to_time_signature(
        old_time_signature_duration, factor=factor)

    # scale contents of measures in expr
    measuretools.scale_contents_of_measures_in_expr(measure, new_time_signature.implied_prolation.reciprocal)

    # assign new time signature
    contexttools.detach_time_signature_marks_attached_to_component(measure)
    new_time_signature.attach(measure)

    # return measure
    return measure



from abjad.tools import contexttools
from abjad.tools import durationtools


def tempo_scaled_duration_to_milliseconds(duration, tempo):
    '''Return the millisecond value of `duration` at `tempo`.

    ::

        >>> duration = (1, 4)
        >>> tempo = contexttools.TempoMark((1, 4), 60)
        >>> quantizationtools.tempo_scaled_duration_to_milliseconds(
        ...     duration, tempo)
        Duration(1000, 1)

    Return ``Duration`` instance.
    '''

    duration = durationtools.Duration(duration)
    tempo = contexttools.TempoMark(tempo)

    whole_note_duration = 1000 \
        * durationtools.Multiplier(tempo.duration.denominator, tempo.duration.numerator) \
        * durationtools.Multiplier(60, tempo.units_per_minute)

    return durationtools.Duration(duration * whole_note_duration)



import abc
from abjad.tools.pitchtools.NumberedObject import NumberedObject
from abjad.tools.pitchtools.PitchClassObject import PitchClassObject


class NumberedPitchClassObject(PitchClassObject, NumberedObject):
    '''.. versionadded:: 2.0

    Numbered pitch-class base class.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    __slots__ = ()

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        pass



from abjad.tools.datastructuretools.TreeNode import TreeNode


class ReSTHeading(TreeNode):
    '''An ReST Heading:

    ::

        >>> heading = documentationtools.ReSTHeading(level=2, text='Section A')
        >>> heading
        ReSTHeading(
            level=2,
            text='Section A'
            )

    ::

        >>> print heading.rest_format
        Section A
        =========

    Return `ReSTHeading` instance.
    '''

    ### INITIALIZER ###

    def __init__(self, level=0, name=None, text=''):
        TreeNode.__init__(self, name=name)
        self.level = level
        self.text = text

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _rest_format_contributions(self):
        result = []
        underline, overline = self.heading_characters[self.level] 
        if overline:
            result.append(overline * len(self.text))
        result.append(self.text)
        result.append(underline * len(self.text))
        return result

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def heading_characters(self):
        return (
            ('#', '#'),
            ('*', '*'),
            ('=', None),
            ('-', None),
            ('^', None),
            ('"', None),
        )

    @property
    def rest_format(self):
        return '\n'.join(self._rest_format_contributions)

    ### READ/WRITE PUBLIC PROPERTIES ###

    @apply
    def level():
        def fget(self):
            return self._level
        def fset(self, arg):
            assert isinstance(arg, int) and \
                0 <= arg < len(self.heading_characters)
            self._level = arg
        return property(**locals())

    @apply
    def text():
        def fget(self):
            return self._text
        def fset(self, arg):
            assert isinstance(arg, str)
            arg = arg.strip()
            assert len(arg)
            self._text = arg
        return property(**locals())




from abjad.tools import leaftools
from abjad.tools import spannertools


def apply_tie_spanner_to_leaf_pair(left, right):
    r'''Apply tie spanner to `left` leaf and `right` leaf::

        >>> staff = Staff("c'8 ~ c' c' c'")

    ::

        >>> f(staff)
        \new Staff {
            c'8 ~
            c'8
            c'8
            c'8
        }

    ::

        >>> tietools.apply_tie_spanner_to_leaf_pair(staff[1], staff[2])

    ::

        >>> f(staff)
        \new Staff {
            c'8 ~
            c'8 ~
            c'8
            c'8
        }

    Handle existing tie spanners intelligently.

    Return none.

    .. versionchanged:: 2.0
        renamed ``tietools.span_leaf_pair()`` to
        ``tietools.apply_tie_spanner_to_leaf_pair()``.
    '''
    from abjad.tools import tietools

    # check input
    assert isinstance(left, leaftools.Leaf)
    assert isinstance(right, leaftools.Leaf)

    # do nothing if leaves are already tied
    if tietools.are_components_in_same_tie_spanner([left, right]):
        return

    # do nothing if leaves are already effectively tied because of parents somewhere in score tree
    left_parent_ties = spannertools.get_spanners_attached_to_any_proper_parent_of_component(
        left, klass=tietools.TieSpanner)
    right_parent_ties = spannertools.get_spanners_attached_to_any_proper_parent_of_component(
        right, klass=tietools.TieSpanner)
    shared_parent_ties = set(left_parent_ties) & set(right_parent_ties)
    if shared_parent_ties:
        return

    # get any left tie spanner
    try:
        left_tie_spanner = spannertools.get_the_only_spanner_attached_to_component(
            left, tietools.TieSpanner)
    except MissingSpannerError:
        left_tie_spanner = None
    
    # get any right tie spanner
    try:
        right_tie_spanner = spannertools.get_the_only_spanner_attached_to_component(
            right, tietools.TieSpanner)
    except MissingSpannerError:
        right_tie_spanner = None

    # fuse or apply tie spanner as appropriate
    if left_tie_spanner is not None and right_tie_spanner is not None:
        left_tie_spanner.fuse(right_tie_spanner)
    elif left_tie_spanner is not None and right_tie_spanner is None:
        left_tie_spanner.append(right)
    elif left_tie_spanner is None and right_tie_spanner is not None:
        right_tie_spanner.append_left(left)
    elif left_tie_spanner is None and right_tie_spanner is None:
        tietools.TieSpanner([left, right])



from abjad.tools import markuptools


def make_reference_manual_lilypond_file(music=None):
    r'''.. versionadded:: 2.9

    Make reference manual LilyPond file.

        >>> score = Score([Staff('c d e f')])
        >>> lilypond_file = documentationtools.make_reference_manual_lilypond_file(score)

    ::

        >>> f(lilypond_file) # doctest: +SKIP

        \version "2.15.37"
        \language "english"
        \include "/Users/trevorbaca/Documents/abjad/trunk/abjad/cfg/abjad.scm"

        \layout {
            indent = #0
            ragged-right = ##t
            \context {
                \Score
                \remove Bar_number_engraver
                \override SpacingSpanner #'strict-grace-spacing = ##t
                \override SpacingSpanner #'strict-note-spacing = ##t
                \override SpacingSpanner #'uniform-stretching = ##t
                \override TupletBracket #'bracket-visibility = ##t
                \override TupletBracket #'minimum-length = #3
                \override TupletBracket #'padding = #2
                \override TupletBracket #'springs-and-rods = #ly:spanner::set-spacing-rods
                \override TupletNumber #'text = #tuplet-number::calc-fraction-text
                proportionalNotationDuration = #(ly:make-moment 1 32)
                tupletFullLength = True
            }
        }

        \paper {
            left-margin = 1.0\in
        }

        \score {
            \new Score <<
                \new Staff {
                    c4
                    d4
                    e4
                    f4
                }
            >>
        }

    Return LilyPond file.
    '''
    from abjad.tools import lilypondfiletools
    from abjad.tools import schemetools

    lilypond_file = lilypondfiletools.make_basic_lilypond_file(music=music)

    # header
    lilypond_file.header_block.tagline = markuptools.Markup('""')

    # layout
    lilypond_file.layout_block.indent = 0
    lilypond_file.line_width = lilypondfiletools.LilyPondDimension(6, 'in')
    lilypond_file.layout_block.ragged_right = True

    # paper
    lilypond_file.paper_block.left_margin = lilypondfiletools.LilyPondDimension(1, 'in')
    
    # score context
    context_block = lilypondfiletools.ContextBlock()
    context_block.context_name = 'Score'
    context_block.engraver_removals.append('Bar_number_engraver')
    context_block.override.spacing_spanner.strict_grace_spacing = True
    context_block.override.spacing_spanner.strict_note_spacing = True
    context_block.override.spacing_spanner.uniform_stretching = True
    context_block.override.tuplet_bracket.bracket_visibility = True
    context_block.override.tuplet_bracket.padding = 2
    context_block.override.tuplet_bracket.springs_and_rods = schemetools.Scheme(
        'ly:spanner::set-spacing-rods')
    context_block.override.tuplet_bracket.minimum_length = 3
    context_block.override.tuplet_number.text = schemetools.Scheme('tuplet-number::calc-fraction-text')
    context_block.set.proportionalNotationDuration = schemetools.SchemeMoment((1, 32))
    context_block.set.tupletFullLength = True
    lilypond_file.layout_block.context_blocks.append(context_block)

    # etc    
    lilypond_file.file_initial_system_comments[:] = []
    lilypond_file.global_staff_size = 12

    return lilypond_file



def detach_key_signature_marks_attached_to_component(component):
    r'''.. versionadded:: 2.3

    Detach key signature marks attached to `component`::

        >>> staff = Staff("c'4 d'4 e'4 f'4")
        >>> key_signature_mark = contexttools.KeySignatureMark('c', 'major')
        >>> key_signature_mark.attach(staff)
        KeySignatureMark(NamedChromaticPitchClass('c'), Mode('major'))(Staff{4})

    ::

        >>> f(staff)
        \new Staff {
            \key c \major
            c'4
            d'4
            e'4
            f'4
        }

    ::

        >>> contexttools.detach_key_signature_marks_attached_to_component(staff)
        (KeySignatureMark(NamedChromaticPitchClass('c'), Mode('major')),)

    ::

        >>> f(staff)
        \new Staff {
            c'4
            d'4
            e'4
            f'4
        }

    Return tuple of zero or more key signature marks.
    '''
    from abjad.tools import contexttools

    marks = []
    for mark in contexttools.get_key_signature_marks_attached_to_component(component):
        mark.detach()
        marks.append(mark)

    return tuple(marks)



from abjad import *


def test_componenttools_get_improper_contents_of_component_01():

    staff = Staff("c' d' e' f'")

    assert componenttools.get_improper_contents_of_component(staff) == [staff] + staff[:]



from abjad.tools import mathtools


def is_integer_equivalent_pair(expr):
    r'''.. versionadded:: 2.9

    True when `expr` is a pair of integer-equivalent expressions::

        >>> sequencetools.is_integer_equivalent_pair((2.0, '3'))
        True

    Otherwise false::

        >>> sequencetools.is_integer_equivalent_pair((2.5, '3'))
        False

    Return boolean.
    '''

    return isinstance(expr, tuple) and len(expr) == 2 and all([
        mathtools.is_integer_equivalent_expr(x) for x in expr])



from abjad.tools import mathtools


def is_integer_equivalent_singleton(expr):
    r'''.. versionadded:: 2.9

    True when `expr` is a singleton of integer-equivalent expressions::
        
        >>> sequencetools.is_integer_equivalent_singleton((2.0,))
        True

    Otherwise false::

        >>> sequencetools.is_integer_equivalent_singleton((2.5,))
        False

    Return boolean.
    '''

    return isinstance(expr, tuple) and len(expr) == 1 and all([
        mathtools.is_integer_equivalent_expr(x) for x in expr])



def get_improper_contents_of_component(component):
    r'''.. versionadded:: 2.9

    Get improper contents of `component`::

        >>> staff = Staff("c' d' e' f'")

    ::

        >>> f(staff)
        \new Staff {
            c'4
            d'4
            e'4
            f'4
        }

    ::

        >>> componenttools.get_improper_contents_of_component(staff)
        [Staff{4}, Note("c'4"), Note("d'4"), Note("e'4"), Note("f'4")]

    The functions works for both containers and leaves.

    Return a list of `component` together with the proper contents of `component`.
    '''

    # initialize result
    result = []

    # append component
    result.append(component)

    # extend proper contents of component
    result.extend(getattr(component, 'music', []))

    # return result
    return result



def spell_chromatic_pitch_number(chromatic_pitch_number, diatonic_pitch_class_name):
    '''.. versionadded:: 1.1

    Spell `chromatic_pitch_number` according to `diatonic_pitch_class_name`::

        >>> pitchtools.spell_chromatic_pitch_number(14, 'c')
        (Accidental('ss'), 5)

    Return accidental / octave-number pair.

    .. versionchanged:: 2.0
        renamed ``pitchtools.number_letter_to_accidental_octave()`` to
        ``pitchtools.spell_chromatic_pitch_number()``.
    '''
    from abjad.tools import pitchtools

    # check input
    if not isinstance(chromatic_pitch_number, (int, long, float)):
        raise TypeError

    if not isinstance(diatonic_pitch_class_name, str):
        raise TypeError

    if not diatonic_pitch_class_name in ['c', 'd', 'e', 'f', 'g', 'a', 'b']:
        raise ValueError

    # find accidental semitones
    pc = pitchtools.diatonic_pitch_class_name_to_chromatic_pitch_class_number(diatonic_pitch_class_name)
    nearest_neighbor = pitchtools.transpose_chromatic_pitch_class_number_to_chromatic_pitch_number_neighbor(
        chromatic_pitch_number, pc)
    semitones = chromatic_pitch_number - nearest_neighbor

    # find accidental alphabetic string
    alphabetic_accidental_abbreviation = \
        pitchtools.Accidental._semitones_to_alphabetic_accidental_abbreviation[semitones]
    accidental = pitchtools.Accidental(alphabetic_accidental_abbreviation)

    # find octave
    octave = pitchtools.chromatic_pitch_number_and_accidental_semitones_to_octave_number(
        chromatic_pitch_number, semitones)

    # return accidental and octave
    return accidental, octave



# -*- encoding: utf-8 -*-
import itertools
from ply import lex
from ply import yacc
from abjad.tools import abctools
from abjad.tools import beamtools
from abjad.tools import chordtools
from abjad.tools import componenttools
from abjad.tools import containertools
from abjad.tools import contexttools
from abjad.tools import durationtools
from abjad.tools import gracetools
from abjad.tools import lilypondfiletools
from abjad.tools import marktools
from abjad.tools import scoretools
from abjad.tools import sequencetools
from abjad.tools import spannertools
from abjad.tools import stafftools
from abjad.tools import tietools
from abjad.tools import voicetools
from abjad.tools.lilypondparsertools._parse import _parse
from abjad.tools.lilypondparsertools._parse_debug import _parse_debug


# apply monkey patch
yacc.LRParser._lilypond_patch_parse = _parse
yacc.LRParser._lilypond_patch_parse_debug = _parse_debug


class LilyPondParser(abctools.Parser):
    r'''Parses a subset of LilyPond input syntax:

    ::

        >>> from abjad.tools.lilypondparsertools import LilyPondParser

    ::

        >>> parser = LilyPondParser()
        >>> input = r"\new Staff { c'4 ( d'8 e' fs'2) \fermata }"
        >>> result = parser(input)
        >>> f(result)
        \new Staff {
            c'4 (
            d'8
            e'8
            fs'2 -\fermata )
        }

    LilyPondParser defaults to English note names, but any of the other
    languages supported by LilyPond may be used:

    ::

        >>> parser = LilyPondParser('nederlands')
        >>> input = '{ c des e fis }'
        >>> result = parser(input)
        >>> f(result)
        {
            c4
            df4
            e4
            fs4
        }

    Briefly, LilyPondParser understands theses aspects of LilyPond syntax:

    - Notes, chords, rests, skips and multi-measure rests
    - Durations, dots, and multipliers
    - All pitchnames, and octave ticks
    - Simple markup (i.e. ``c'4 ^ "hello!"``)
    - Most articulations
    - Most spanners, including beams, slurs, phrasing slurs, ties, and glissandi
    - Most context types via ``\new`` and ``\context``, 
      as well as context ids (i.e. ``\new Staff = "foo" { }``)
    - Variable assignment (i.e. ``global = { \time 3/4 } \new Staff { \global }``)
    - Many music functions:
        - ``\acciaccatura``
        - ``\appoggiatura``
        - ``\bar``
        - ``\breathe``
        - ``\clef``
        - ``\grace``
        - ``\key``
        - ``\transpose``
        - ``\language``
        - ``\makeClusters``
        - ``\mark``
        - ``\oneVoice``
        - ``\relative``
        - ``\skip``
        - ``\slashedGrace``
        - ``\time``
        - ``\times``
        - ``\transpose``
        - ``\voiceOne``, ``\voiceTwo``, ``\voiceThree``, ``\voiceFour``

    LilyPondParser currently **DOES NOT** understand many other aspects of LilyPond syntax:

    - ``\markup``
    - ``\book``, ``\bookpart``, ``\header``, ``\layout``, ``\midi`` and ``\paper``
    - ``\repeat`` and ``\alternative``
    - Lyrics
    - ``\chordmode``, ``\drummode`` or ``\figuremode``
    - Property operations, such as ``\override``, 
      ``\revert``, ``\set``, ``\unset``, and ``\once``
    - Music functions which generate or extensively mutate musical structures
    - Embedded Scheme statements (anything beginning with ``#``)

    Returns LilyPondParser instance.
    '''

    def __init__(self, default_language='english', debug=False):

        from abjad.tools import lilypondparsertools

        from abjad.ly.current_module import current_module
        from abjad.ly.language_pitch_names import language_pitch_names
        from abjad.ly.markup_functions import markup_functions
        from abjad.ly.markup_functions import markup_list_functions

        # LilyPond emulation data
        self._guile = lilypondparsertools.GuileProxy(self)
        self._current_module = current_module
        self._language_pitch_names = language_pitch_names
        self._markup_functions = markup_functions
        self._markup_list_functions = markup_list_functions
        self.default_language = default_language

        # attach parser and lexer rules
        self._lexdef = lilypondparsertools.LilyPondLexicalDefinition(self)
        self._syndef = lilypondparsertools.LilyPondSyntacticalDefinition(self)

        # build PLY parser and lexer
        abctools.Parser.__init__(self, debug=debug)

        self._reset_parser_variables()

    ### SPECIAL METHODS ###

    def __call__(self, input_string):

        self._reset_parser_variables()

        if self._debug:
            result = self._parser._lilypond_patch_parse_debug(
                input_string, 
                lexer=self._lexer,
                debug=self._logger)
        else:
            result = self._parser._lilypond_patch_parse(
                input_string,
                lexer=self._lexer)

        if isinstance(result, containertools.Container):
            self._apply_spanners(result)
        elif isinstance(result, lilypondfiletools.LilyPondFile):
            for x in result:
                if isinstance(x, containertools.Container):
                    self._apply_spanners(x)
                elif isinstance(x, lilypondfiletools.ScoreBlock):
                    for y in x:
                        self._apply_spanners(y)

        return result


    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def available_languages(self):
        '''Tuple of pitch-name languages supported by LilyPondParser:

        ::

            >>> from abjad.tools.lilypondparsertools import LilyPondParser
            >>> parser = LilyPondParser()
            >>> for language in parser.available_languages:
            ...     print language
            catalan
            deutsch
            english
            espanol
            español
            français
            italiano
            nederlands
            norsk
            portugues
            suomi
            svenska
            vlaams

        Return tuple.
        '''
        return tuple(sorted(self._language_pitch_names.keys()))

    @property
    def lexer_rules_object(self):
        return self._lexdef

    @property
    def parser_rules_object(self):
        return self._syndef

    ### READ/WRITE PUBLIC PROPERTIES ###

    @apply
    def default_language():
        def fget(self):
            '''Read/write attribute to set parser's default pitch-name language:

            ::

                >>> from abjad.tools.lilypondparsertools import LilyPondParser

            ::

                >>> parser = LilyPondParser()

            ::

                >>> parser.default_language
                'english'

            ::

                >>> parser('{ c df e fs }')
                {c4, df4, e4, fs4}

            ::

                >>> parser.default_language = 'nederlands'
                >>> parser.default_language
                'nederlands'

            ::

                >>> parser('{ c des e fis }')
                {c4, df4, e4, fs4}

            Return string.
            '''
            return self._default_language
        def fset(self, arg):
            assert arg in self.available_languages
            self._default_language = arg
        return property(**locals())

    ### PRIVATE METHODS ###

    def _apply_spanners(self, music):

        # get local reference to methods
        _get_span_events = self._get_span_events
        _span_event_name_to_spanner_class = self._span_event_name_to_spanner_class

        # dictionary of spanner classes and instances
        all_spanners = { }

        # traverse all leaves
        leaves = music.leaves
        first_leaf = None
        if leaves:
            first_leaf = leaves[0]
        for leaf, next_leaf in sequencetools.iterate_sequence_pairwise_wrapped(leaves):

            span_events = _get_span_events(leaf)
            directed_events = { }

            # sort span events into directed and undirected groups
            for span_event in span_events:
                klass = _span_event_name_to_spanner_class(span_event.name)

                # group directed span events by their Abjad spanner class
                if hasattr(span_event, 'span_direction'):
                    if klass not in directed_events:
                        directed_events[klass] = [span_event]
                    else:
                        directed_events[klass].append(span_event)
                    if klass not in all_spanners:
                        all_spanners[klass] = []

                # or apply undirected event immediately (i.e. ties, glisses)
                elif next_leaf is not first_leaf: # so long as we are not wrapping yet
                    previous_spanners = [x for x in leaf.spanners if isinstance(x, klass)]
                    if previous_spanners:
                        previous_spanners[0].append(next_leaf)
                    else:
                        if hasattr(span_event, 'direction') and hasattr(klass, 'direction'):
                            klass([leaf, next_leaf], direction=span_event.direction)
                        else:
                            klass([leaf, next_leaf])

                # otherwise throw an error
                else:
                    raise Exception('Unterminated %s at %s.' % (klass.__name__, leaf))

            # check for DynamicMarks, and terminate any hairpin
            dynamics = contexttools.get_dynamic_marks_attached_to_component(leaf)
            if dynamics and spannertools.HairpinSpanner in all_spanners and \
                all_spanners[spannertools.HairpinSpanner]:
                all_spanners[spannertools.HairpinSpanner][0].append(leaf)
                all_spanners[spannertools.HairpinSpanner].pop()

            # loop through directed events, handling each as necessary
            for klass, events in directed_events.iteritems():

                starting_events, stopping_events = [], []
                for x in events:
                    if x.span_direction == 'start':
                        starting_events.append(x)
                    else:
                        stopping_events.append(x)

                if klass is beamtools.BeamSpanner:
                    # A beam may begin and end on the same leaf
                    # but only one beam spanner may cover any given leaf,
                    # and starting events are processed before ending ones
                    for event in starting_events:
                        if all_spanners[klass]:
                            raise Exception('Already have beam.')
                        if hasattr(event, 'direction'):
                            all_spanners[klass].append(klass(direction=event.direction))
                        else:
                            all_spanners[klass].append(klass())
                    for _ in stopping_events:
                        if all_spanners[klass]:
                            all_spanners[klass][0].append(leaf)
                            all_spanners[klass].pop()

                elif klass is spannertools.HairpinSpanner:
                    # Dynamic events can be ended many times,
                    # but only one may start on a given leaf,
                    # and the event must start and end on separate leaves.
                    # If a hairpin already exists and another starts,
                    # the pre-existant spanner is ended.
                    for _ in stopping_events:
                        if all_spanners[klass]:
                            all_spanners[klass][0].append(leaf)
                            all_spanners[klass].pop()
                    if 1 == len(starting_events):
                        if all_spanners[klass]:
                            all_spanners[klass][0].append(leaf)
                            all_spanners[klass].pop()
                        shape = '<'
                        event = starting_events[0]
                        if event.name == 'DecrescendoEvent':
                            shape = '>'
                        if hasattr(event, 'direction'):
                            all_spanners[klass].append(klass([], shape, direction=event.direction))
                        else:
                            all_spanners[klass].append(klass([], shape))
                    elif 1 < len(starting_events):
                        raise Exception('Simultaneous dynamic-span events.')
                    
                elif klass in [spannertools.SlurSpanner, spannertools.PhrasingSlurSpanner,
                    spannertools.TextSpanner, spannertools.TrillSpanner]:
                    # These engravers process stop events before start events,
                    # they must contain more than one leaf,
                    # however, they can stop on a leaf and start on the same leaf.
                    for _ in stopping_events:                    
                        if all_spanners[klass]:
                            all_spanners[klass][0].append(leaf)
                            all_spanners[klass].pop()
                        else:
                            raise Exception('Cannot end %s.' % klass.__name__)
                    for event in starting_events:
                        if not all_spanners[klass]:
                            if hasattr(event, 'direction') and hasattr(klass, 'direction'):
                                all_spanners[klass].append(klass(direction=event.direction))
                            else:
                                all_spanners[klass].append(klass())
                        else:
                            raise Exception('Already have %s.' % klass.__name__)

                elif klass is spannertools.HorizontalBracketSpanner:
                    # Brackets can nest, meaning
                    # multiple brackets can begin or end on a leaf
                    # but cannot both begin and end on the same leaf
                    # and therefore a bracket cannot cover a single leaf
                    has_starting_events = bool(len(starting_events))
                    for _ in starting_events:
                        all_spanners[klass].append(klass())
                    if stopping_events:
                        if not has_starting_events:
                            for _ in stopping_events:
                                if all_spanners[klass]:
                                    all_spanners[klass][-1].append(leaf)
                                    all_spanners[klass].pop()
                                else:
                                    raise Exception('Do not have that many brackets.')
                        else:
                            raise Exception('Conflicting note group events.')

            # append leaf to all tracked spanners,
            for klass, instances in all_spanners.iteritems():
                for instance in instances:
                    instance.append(leaf)

        # check for unterminated spanners
        for klass, instances in all_spanners.iteritems():
            if instances:
                raise Exception('Unterminated %s.' % klass.__name__)

    def _assign_variable(self, identifier, value):
        self._scope_stack[-1][identifier] = value

    def _backup_token(self, token_type, token_value):
        if self._debug:
            self._logger.info('Extra  : Backing up')

        # push the current lookahead back onto the lookaheadstack
        self._push_extra_token(self._parser.lookahead)

        # create the backup token, set as new lookahead
        backup = lex.LexToken()
        backup.type = 'BACKUP'
        backup.value = '(backed-up?)'
        backup.lexpos = 0
        backup.lineno = 0
        self._parser.lookahead = backup

        if token_type:
            token = lex.LexToken()
            token.type = token_type
            token.value = token_value
            token.lexpos = 0
            token.lineno = 0
            self._push_extra_token(token)

    def _construct_context_specced_music(self, context, optional_id, optional_context_mod, music):
        known_contexts = {
            'ChoirStaff': scoretools.StaffGroup,
            'GrandStaff': scoretools.GrandStaff,
            'PianoStaff': scoretools.PianoStaff,
            'Score': scoretools.Score,
            'Staff': stafftools.Staff,
            'StaffGroup': scoretools.StaffGroup,
            'Voice': voicetools.Voice,
        }
        if context in known_contexts:
            context = known_contexts[context]([])
        else:
            raise Exception('Context type %s not supported.' % context)

        if optional_id is not None:
            context.name = optional_id        

        if optional_context_mod is not None:
            for x in optional_context_mod:
                print x
            pass # TODO: Implement context mods on contexts. #

        context.is_parallel = music.is_parallel

        # add children
        while len(music):
            component = music.pop(0)
            context.append(component)

        marks = music._marks_for_which_component_functions_as_start_component
        for mark in marks:
            mark(context)

        return context

    def _construct_sequential_music(self, music):
        # mark sorting could be rewritten into a single list, using tuplets,
        # with t[0] being 'forward' or 'backward' and t[1] being the mark, as
        # this better preserves attachment order.  Not clear if we need it.

        container = containertools.Container()
        previous_leaf = None
        apply_forward = []
        apply_backward = []

        # sort events into forward or backwards attaching, and attach them to
        # the proper leaf
        for x in music:
            if isinstance(x, componenttools.Component) and not isinstance(x, gracetools.GraceContainer):
                for mark in apply_forward:
                    if hasattr(mark, '__call__'):
                        mark(x)
                if previous_leaf:
                    for mark in apply_backward:
                        if hasattr(mark, '__call__'):
                            mark(previous_leaf)
                else:
                    for mark in apply_backward:
                        if hasattr(mark, '__call__'):
                            mark.format_slot = 'before'
                            mark(x)
                apply_forward = []
                apply_backward = []
                previous_leaf = x
                container.append(x)
            else:
                if isinstance(x, marktools.BarLine):
                    apply_backward.append(x)
                elif isinstance(x, marktools.LilyPondCommandMark) and \
                    x.command_name in ['breathe']:
                        apply_backward.append(x)
                else:
                    apply_forward.append(x)

        # attach remaining events to last leaf, or to the container itself if
        # there were no leaves
        if previous_leaf:
            for mark in apply_forward:
                if hasattr(mark, '__call__'):
                    mark.format_slot = 'after'
                    mark(previous_leaf)
            for mark in apply_backward:
                if hasattr(mark, '__call__'):
                    mark(previous_leaf)
        else:
            for mark in apply_forward:
                if hasattr(mark, '__call__'):
                    mark.format_slot = 'opening'
                    mark(container)
            for mark in apply_backward:
                if hasattr(mark, '__call__'):
                    mark.format_slot = 'opening'
                    mark(container)

        return container

    def _construct_simultaneous_music(self, music):
        from abjad.tools import lilypondparsertools
        def is_separator(x):
            if isinstance(x, lilypondparsertools.LilyPondEvent):
                if x.name == 'VoiceSeparator':
                    return True
            return False
        
        container = containertools.Container()
        container.is_parallel = True

        # check for voice separators     
        groups = []
        for value, group in itertools.groupby(music, is_separator):
            if not value:
                groups.append(list(group))

        # without voice separators
        if 1 == len(groups):
            assert all([isinstance(x, contexttools.Context) for x in groups[0]])
            container.extend(groups[0])
        # with voice separators
        else:
            for group in groups:
                container.append(Voice(self._construct_sequential_music(group)[:]))

        return container

    @classmethod
    def _get_scheme_predicates(cls):
        return {
            'boolean?':           lambda x: isinstance(x, bool),
            'cheap-list?':        lambda x: isinstance(x, (list, tuple)),
            'cheap-markup?':      lambda x: isinstance(x, markuptools.MarkupCommand),
            'fraction?':          lambda x: isinstance(x, lilypondparsertools.LilyPondFraction),
            'integer?':           lambda x: isinstance(x, int),
            'list?':              lambda x: isinstance(x, (list, tuple)),
            'ly:duration?':       lambda x: isinstance(x, lilypondparsertools.LilyPondDuration),
            'ly:music?':          lambda x: isinstance(x, (componenttools.Component, marktools.Mark)),
            'ly:pitch?':          lambda x: isinstance(x, pitchtools.NamedChromaticPitch),
            'markup?':            lambda x: isinstance(x, markuptools.MarkupCommand),
            'number-list?':       lambda x: isinstance(x, (list, tuple)) and \
                                            all([isinstance(y, (int, float)) for y in x]),
            'number?':            lambda x: isinstance(x, (int, float)),
            'real?':              lambda x: isinstance(x, (int, float)),
            'string?':            lambda x: isinstance(x, (str, unicode)),
            'void?':              lambda x: isinstance(x, type(None)),
            # the following predicates have not yet been implemented in Abjad
            'hash-table?':        lambda x: True,
            'list-or-symbol?':    lambda x: True,
            'ly:dir?':            lambda x: True,
            'ly:moment?':         lambda x: True,
            'number-or-string?':  lambda x: True,
            'number-pair?':       lambda x: True,
            'optional?':          lambda x: True,
            'pair?':              lambda x: True,
            'procedure?':         lambda x: True,
            'scheme?':            lambda x: True,
            'string-or-pair?':    lambda x: True,
            'symbol-or-boolean?': lambda x: True,
            'symbol?':            lambda x: True,
        }

    def _get_span_events(self, leaf):
        annotations = marktools.get_annotations_attached_to_component(leaf)
        if annotations:
            spanners_annotations = [x for x in annotations if x.name == 'spanners']
            if 1 == len(spanners_annotations):
                return spanners_annotations[0].value
            elif 1 < len(spanners_annotations):
                raise Exception('Multiple span events lists attached to %s' % leaf)
        return []

    def _process_post_events(self, leaf, post_events):
        for post_event in post_events:
            if hasattr(post_event, '__call__'):
                post_event(leaf)
            else:
                annotation = [x for x in marktools.get_annotations_attached_to_component(leaf)
                    if x.name == 'spanners']
                if not annotation:
                    annotation = marktools.Annotation('spanners', [])(leaf)
                else:
                    annotation = annotation[0]
                annotation.value.append(post_event)

    def _push_variable_scope(self):
        self._scope_stack.append({})

    def _pop_variable_scope(self):
        if self._scope_stack:
            self._scope_stack.pop()

    def _push_extra_token(self, token):
        self._parser.lookaheadstack.append(token)

    def _relex_lookahead(self):
        if not str(self._parser.lookahead) == '$end':
            difference = self._parser.lookahead.lexpos - self._lexer.lexpos
            self._lexer.skip(difference)
            self._parser.lookahead = None

    def _reparse_token(self, predicate, token_type, token_value):
        if self._debug:
            self._logger.info('Extra  : Reparsing')

        # push the current lookahead back onto the lookaheadstack
        self._push_extra_token(self._parser.lookahead)

        token = lex.LexToken()
        token.type = token_type
        token.value = token_value
        token.lexpos = 0
        token.lineno = 0
        self._push_extra_token(token)

        reparse = lex.LexToken()
        reparse.type = 'REPARSE'
        reparse.value = predicate
        reparse.lineno = 0
        reparse.lexpos = 0
        self._parser.lookahead = reparse

    def _reset_parser_variables(self):
        from abjad.tools import lilypondparsertools
        try:
            self._parser.restart()
        except:
            pass
        self._scope_stack = [{}]
        self._chord_pitch_orders = {}
        self._lexer.push_state('notes')
        self._default_duration = lilypondparsertools.LilyPondDuration(durationtools.Duration(1, 4), None)
        self._last_chord = chordtools.Chord(['c', 'g', "c'"], (1, 4)) # LilyPond's default!
        self._pitch_names = self._language_pitch_names[self.default_language]
        self._repeated_chords = {}

    def _resolve_identifier(self, identifier):
        for scope in reversed(self._scope_stack):
            if identifier in scope:
                return scope[identifier]
        return None

    def _resolve_event_identifier(self, identifier):
        from abjad.tools import lilypondparsertools
        lookup = self._current_module[identifier] # without any leading slash
        name = lookup['name']
        if name == 'ArticulationEvent':
            return marktools.Articulation(lookup['articulation-type'])
        elif name == 'AbsoluteDynamicEvent':
            return contexttools.DynamicMark(lookup['text'])
        elif name == 'LaissezVibrerEvent':
            return marktools.LilyPondCommandMark('laissezVibrer', 'after')
        event = lilypondparsertools.LilyPondEvent(name)
        if 'span-direction' in lookup:
            if lookup['span-direction'] == -1:
                event.span_direction = 'start'
            else:
                event.span_direction = 'stop'
        return event

    def _span_event_name_to_spanner_class(self, name):
        spanners = {
            'BeamEvent': beamtools.BeamSpanner,
            'CrescendoEvent': spannertools.HairpinSpanner,
            'DecrescendoEvent': spannertools.HairpinSpanner,
            'GlissandoEvent': spannertools.GlissandoSpanner,
            'NoteGroupingEvent': spannertools.HorizontalBracketSpanner,
            'PhrasingSlurEvent': spannertools.PhrasingSlurSpanner,
            'SlurEvent': spannertools.SlurSpanner,
            'TextSpanEvent': spannertools.TextSpanner,
            'TieEvent': tietools.TieSpanner,
            'TrillSpanEvent': spannertools.TrillSpanner,
        }
        if name in spanners:
            return spanners[name]
        raise Exception('Abjad cannot associate a spanner class with %s' % name)

    def _test_scheme_predicate(self, predicate, value):
        from abjad.tools import lilypondparsertools
        predicates = self._get_scheme_predicates()
        if predicate in predicates:
            return predicates[predicate](value)
        return True

    ### PUBLIC METHODS ###

    @classmethod
    def register_markup_function(cls, name, signature):
        r'''Register a custom markup function globally with LilyPondParser:

        ::

            >>> from abjad.tools.lilypondparsertools import LilyPondParser

        ::

            >>> name = 'my-custom-markup-function'
            >>> signature = ['markup?']
            >>> LilyPondParser.register_markup_function(name, signature)

        ::

            >>> parser = LilyPondParser()
            >>> string = r"\markup { \my-custom-markup-function { foo bar baz } }"
            >>> parser(string)
            Markup((MarkupCommand('my-custom-markup-function', ['foo', 'bar', 'baz']),))

        `signature` should be a sequence of zero or more type-predicate names, as
        understood by LilyPond.  Consult LilyPond's documentation for a complete
        list of all understood type-predicates.

        Return None
        '''

        from abjad.ly.markup_functions import markup_functions

        assert isinstance(name, str)
        assert all([not x.isspace() for x in name])
        assert isinstance(signature, (list, tuple))
        for predicate in signature:
            assert isinstance(predicate, str)
            assert all([not x.isspace() for x in predicate])
            assert predicate.endswith('?')

        markup_functions[name] = tuple(signature)



def list_time_signatures_of_measures_in_expr(components):
    r'''.. versionadded:: 2.0

    List time signatures of measures in `expr`::

        >>> staff = Staff('abj: | 2/8 c8 d8 || 3/8 c8 d8 e8 || 4/8 c8 d8 e8 f8 |')

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c8
                d8
            }
            {
                \time 3/8
                c8
                d8
                e8
            }
            {
                \time 4/8
                c8
                d8
                e8
                f8
            }
        }

    ::

        >>> for x in measuretools.list_time_signatures_of_measures_in_expr(staff): x
        ...
        TimeSignatureMark((2, 8))(|2/8, c8, d8|)
        TimeSignatureMark((3, 8))(|3/8, c8, d8, e8|)
        TimeSignatureMark((4, 8))(|4/8, c8, d8, e8, f8|)

    Return list of zero or more time signatures.

    .. versionchanged:: 2.0
        renamed ``measuretools.list_time_signatures_of_mesures_in_expr()`` to
        ``measuretools.list_time_signatures_of_measures_in_expr()``.
    '''
    from abjad.tools import contexttools
    from abjad.tools import iterationtools

    # create empty list to hold result
    result = []

    # iterate measures and store time signature pairs
    for measure in iterationtools.iterate_measures_in_expr(components):
        time_signature = contexttools.get_effective_time_signature(measure)
        result.append(time_signature)

    # return result
    return result



from abjad.tools import componenttools


def get_spanners_attached_to_any_improper_child_of_component(component, klass=None):
    r'''.. versionadded:: 2.0

    Get all spanners attached to any improper children of `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> beam = beamtools.BeamSpanner(staff.leaves)
        >>> first_slur = spannertools.SlurSpanner(staff.leaves[:2])
        >>> second_slur = spannertools.SlurSpanner(staff.leaves[2:])
        >>> trill = spannertools.TrillSpanner(staff)

    ::

        >>> f(staff)
        \new Staff {
            c'8 [ ( \startTrillSpan
            d'8 )
            e'8 (
            f'8 ] ) \stopTrillSpan
        }

    ::

        >>> len(spannertools.get_spanners_attached_to_any_improper_child_of_component(staff))
        4

    Get all spanners of `klass` attached to any proper children of `component`::

        >>> spanner_klass = spannertools.SlurSpanner
        >>> result = spannertools.get_spanners_attached_to_any_proper_child_of_component(
        ... staff, spanner_klass)

    ::

        >>> list(sorted(result))
        [SlurSpanner(c'8, d'8), SlurSpanner(e'8, f'8)]

    Get all spanners of any `klass` attached to any proper children of `component`::

        >>> spanner_klasses = (spannertools.SlurSpanner, beamtools.BeamSpanner)
        >>> result = spannertools.get_spanners_attached_to_any_proper_child_of_component(
        ... staff, spanner_klasses) 

    ::

        >>> list(sorted(result))
        [BeamSpanner(c'8, d'8, e'8, f'8), SlurSpanner(c'8, d'8), SlurSpanner(e'8, f'8)]

    Return unordered set of zero or more spanners.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import spannertools

    # note: externalization of (old) component spanner aggregator 'contained' property
    result = set([])

    # inspect component itself
    result.update(spannertools.get_spanners_attached_to_component(component, klass))

    # iterate proper children of component
    children = iterationtools.iterate_components_in_expr(component, componenttools.Component)
    for child in children:
        result.update(spannertools.get_spanners_attached_to_component(child, klass))

    # return result
    return result



import os
import sys


# TODO: make public
def _open_file(file_name, application=None):
    '''Generic cross-platform file opener.
    '''
    from abjad.tools import iotools

    if os.name == 'nt':
        os.startfile(file_name)
    else:
        if sys.platform.lower() == 'linux2':
            viewer = application or 'xdg-open'
        else:
            viewer = application or 'open'
        iotools.spawn_subprocess('{} {} &'.format(viewer, file_name))



from abjad import *


def configure_score(score):

    spacing_vector = layouttools.make_spacing_vector(0, 0, 8, 0)
    score.override.vertical_axis_group.staff_staff_spacing = spacing_vector
    score.override.staff_grouper.staff_staff_spacing = spacing_vector
    score.override.staff_symbol.thickness = 0.5
    score.set.mark_formatter = schemetools.Scheme('format-mark-box-numbers')



from abjad.tools.rhythmmakertools.DivisionIncisedRhythmMaker import DivisionIncisedRhythmMaker


class DivisionIncisedRestRhythmMaker(DivisionIncisedRhythmMaker):
    r'''.. versionadded:: 2.8

    Division-incised rest rhythm-maker::

        >>> prefix_talea, prefix_lengths = [8], [1, 2, 3, 4]
        >>> suffix_talea, suffix_lengths = [1], [1]
        >>> talea_denominator = 32
        >>> maker = rhythmmakertools.DivisionIncisedRestRhythmMaker(
        ... prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator)

    ::

        >>> divisions = [(5, 8), (5, 8), (5, 8), (5, 8)]
        >>> leaf_lists = maker(divisions)
        >>> leaves = sequencetools.flatten_sequence(leaf_lists)

    ::

        >>> measures = measuretools.make_measures_with_full_measure_spacer_skips(divisions)
        >>> staff = Staff(measures)
        >>> measures = measuretools.replace_contents_of_measures_in_expr(staff, leaves)

    ::

        >>> f(staff) 
        \new Staff {
            {
                \time 5/8
                c'4
                r4
                r16.
                c'32
            }
            {
                c'4
                c'4
                r16.
                c'32
            }
            {
                c'4
                c'4
                c'8
            }
            {
                c'4
                c'4
                c'8
            }
        }

    Usage follows the two-step instantiate-then-call pattern shown here.

    Return rhythm-maker.
    '''

    ### PRIVATE METHODS ###

    def _make_middle_of_numeric_map_part(self, middle):
        if 0 < middle:
            return (-abs(middle), )
        else:
            return ()



from abjad import *


def apply_expressive_marks(score):

    voice = score['First Violin Voice']
    markuptools.Markup(r'\left-column { div. \line { con sord. } }', Up)(voice[6][1])
    markuptools.Markup('sim.', Up)(voice[8][0])
    markuptools.Markup('uniti', Up)(voice[58][3])
    markuptools.Markup('div.', Up)(voice[59][0])
    markuptools.Markup('uniti', Up)(voice[63][3])

    voice = score['Second Violin Voice']
    markuptools.Markup('div.', Up)(voice[7][0])
    markuptools.Markup('uniti', Up)(voice[66][1])
    markuptools.Markup('div.', Up)(voice[67][0])
    markuptools.Markup('uniti', Up)(voice[74][0])

    voice = score['Viola Voice']
    markuptools.Markup('sole', Up)(voice[8][0])

    voice = score['Cello Voice']
    markuptools.Markup('div.', Up)(voice[10][0])
    markuptools.Markup('uniti', Up)(voice[74][0])
    markuptools.Markup('uniti', Up)(voice[84][1])
    markuptools.Markup(r'\italic { espr. }', Down)(voice[86][0])
    markuptools.Markup(r'\italic { molto espr. }', Down)(voice[88][1])

    voice = score['Bass Voice']
    markuptools.Markup('div.', Up)(voice[14][0])
    markuptools.Markup(r'\italic { espr. }', Down)(voice[86][0])
    componenttools.split_components_at_offsets(voice[88][:], [Duration(1, 1), Duration(1, 2)])
    markuptools.Markup(r'\italic { molto espr. }', Down)(voice[88][1])
    markuptools.Markup('uniti', Up)(voice[99][1])

    for voice in iterationtools.iterate_voices_in_expr(score['Strings Staff Group']):
        markuptools.Markup(r'\italic { (non dim.) }', Down)(voice[102][0])







import collections
import copy
from abjad.tools import abctools
from abjad.tools import durationtools
from abjad.tools.timeintervaltools.TimeIntervalMixin import TimeIntervalMixin


class TimeInterval(TimeIntervalMixin, collections.MutableMapping):
    '''A start / stop pair, carrying some metadata.'''

    __slots__ = ('_data', '_start', '_stop')

    def __init__(self, *args):
        if len(args) == 1 and isinstance(args[0], type(self)):
            start, stop, data = args[0].start, args[0].stop, args[0]
        elif len(args) == 2:
            start, stop, data = args[0], args[1], {}
        elif len(args) == 3:
            start, stop, data = args[0], args[1], args[2]

        start, stop = durationtools.Offset(start), durationtools.Offset(stop)
        assert start < stop
        if isinstance(data, type(self)):
            data = data._data
        assert isinstance(data, dict)

        object.__setattr__(self, '_start', start)
        object.__setattr__(self, '_stop', stop)
        object.__setattr__(self, '_data', copy.copy(data))

    ### SPECIAL METHODS ###

    def __delitem__(self, item):
        self._data.__delitem__(item)

    def __eq__(self, other):
        if type(other) == type(self) and \
            other.start == self.start and \
            other.stop == self.stop and \
            other._data == self._data:
                return True
        return False

    def __getitem__(self, item):
        return self._data.__getitem__(item)

    def __hash__(self):
        return id(self)

    def __iter__(self):
        for x in self._data:
            yield x

    def __len__(self):
        return len(self._data)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return '%s(%r, %r, %r)' % (self._class_name, self.start, self.stop, self._data)

    def __setitem__(self, item, value):
        self._data.__setitem__(item, value)

    ### PUBLIC PROPERTIES ###

    @property
    def center(self):
        '''Center point of start and stop bounds.'''
        return durationtools.Offset(self.stop + self.start) / 2

    @property
    def duration(self):
        '''stop bound minus start bound.'''
        return durationtools.Duration(self.stop - self.start)

    @property
    def signature(self):
        '''Tuple of start bound and stop bound.'''
        return (self.start, self.stop)

    @property
    def start(self):
        '''start bound.'''
        return self._start

    @property
    def stop(self):
        '''stop bound.'''
        return self._stop

    ### PRIVATE METHODS ###

    def _get_tools_package_qualified_repr_pieces(self, is_indented=True):
        pieces = []
        pieces.append('{}('.format(self._tools_package_qualified_class_name))
        pieces.append('\t{!r},'.format(self.start))
        pieces.append('\t{!r},'.format(self.stop))

        if len(self):
            pieces.append('\t{')

            for key, value in self.iteritems():
                if isinstance(key, abctools.AbjadObject):
                    key = key._tools_package_qualified_repr
                else:
                    key = repr(key)
                if isinstance(value, abctools.AbjadObject):
                    vpieces = value._get_tools_package_qualified_repr_pieces()
                    if 1 < len(vpieces):
                        pieces.append('\t\t{}: {}'.format(key, vpieces[0]))
                        for piece in vpieces[1:-1]:
                            pieces.append('\t\t{}'.format(piece))
                        pieces.append('\t\t{},'.format(vpieces[-1]))
                    else:
                        pieces.append('\t\t{}: {},'.format(key, vpieces[0]))
                else:
                    pieces.append('\t\t{}: {!r},'.format(key, value))

            pieces.append('\t})')

        else:
            pieces.append('\t)')
        return pieces

    ### PUBLIC METHODS ###

    def quantize_to_rational(self, rational):
        rational = durationtools.Duration(rational)
        assert 0 < rational
        start = durationtools.Offset(int(round(interval.start / rational))) * rational
        stop = durationtools.Offset(int(round(interval.stop / rational))) * rational
        if start == stop:
            stop = start + rational
        return type(self)(start, stop, self)

    def scale_by_rational(self, rational):
        rational = durationtools.Duration(rational)
        assert 0 < rational
        if rational != 1:
            new_duration = (self.stop - self.start) * rational
            return type(self)(self.start, self.start + new_duration, self)
        else:
            return self

    def scale_to_rational(self, rational):
        rational = durationtools.Duration(rational)
        assert 0 < rational
        if rational != self.duration:
            return type(self)(self.start, self.start + rational, self)
        else:
            return self

    def shift_by_rational(self, rational):
        rational = durationtools.Duration(rational)
        if rational != 0:
            return type(self)(self.start + rational, self.stop + rational, self)
        else:
            return self

    def shift_to_rational(self, rational):
        rational = durationtools.Offset(rational)
        if rational != self.start:
            duration = self.stop - self.start
            return type(self)(rational, rational + duration, self)
        else:
            return self

    def split_at_rationals(self, *rationals):
        rationals = [durationtools.Offset(x) for x in rationals]
        assert 0 < len(rationals)

        intervals = [self]
        new_intervals = []
        for rational in sorted(rationals):
            for interval in intervals:
                if interval.start < rational < interval.stop:
                    new_intervals.append(type(self)(interval.start, rational, self))
                    new_intervals.append(type(self)(rational, interval.stop, self))
                else:
                    new_intervals.append(type(self)(interval))
            intervals = new_intervals
            new_intervals = []

        return tuple(sorted(intervals, key=lambda x: x.signature))



from abjad.tools import notetools


def fill_measures_in_expr_with_minimal_number_of_notes(expr, decrease_durations_monotonically=True, iterctrl=None):
    '''.. versionadded:: 1.1

    Fill measures in `expr` with minimal number of notes that decrease durations monotonically::

        >>> measure = Measure((5, 18), [])

    ::

        >>> measuretools.fill_measures_in_expr_with_minimal_number_of_notes(
        ...     measure, decrease_durations_monotonically=True)

    ::

        >>> f(measure)
        {
            \time 5/18
            \scaleDurations #'(8 . 9) {
                c'4 ~
                c'16
            }
        }

    Fill measures in `expr` with minimal number of notes that increase durations monotonically::

        >>> measure = Measure((5, 18), [])

    ::

        >>> measuretools.fill_measures_in_expr_with_minimal_number_of_notes(
        ...     measure, decrease_durations_monotonically=False)

    ::

        >>> f(measure)
        {
            \time 5/18
            \scaleDurations #'(8 . 9) {
                c'16 ~
                c'4
            }
        }

    Return none.
    '''
    from abjad.tools import contexttools
    from abjad.tools import iterationtools

    if iterctrl is None:
        iterctrl = lambda measure, i: True
    for i, measure in enumerate(iterationtools.iterate_measures_in_expr(expr)):
        if iterctrl(measure, i):
            time_signature = contexttools.get_effective_time_signature(measure)
            written_duration = time_signature.duration / time_signature.implied_prolation
            notes = notetools.make_notes(0, written_duration, decrease_durations_monotonically=decrease_durations_monotonically)
            measure[:] = notes



from abjad.tools import sequencetools


def test_sequencetools_CyclicTree_storage_format_01():

    cyclic_tree = sequencetools.CyclicTree([[1, 2, 3], [4, 5]])

    assert cyclic_tree.storage_format == 'sequencetools.CyclicTree(\n\t[[1, 2, 3], [4, 5]]\n\t)'
